@@ -20,7 +20,7 @@
 ## should create a real icon canvas object that makes all this tk 
 ## code superfluous
 ##
-## Copyright (c) 1993, 1994
+## Copyright (c) 1993, 1994, 1995
 ##
 ## J. Schoenwaelder
 ## TU Braunschweig, Germany
@@ -105,17 +105,11 @@ proc ::font { id type font} {
 }
 
 proc ::raise { id } {
-    set c [$id canvas]
-    set item [$id items]
-    $c raise $item
-    catch {$c lower clip$item label$item}
+    [$id canvas] raise "id $id"
 }
 
 proc ::lower { id } {
-    set c [$id canvas]
-    set item [$id items]
-    $c lower $item
-    catch {$c lower clip$item label$item}
+    [$id canvas] lower "id $id"
 }
 
 proc ::icon { id bitmap } {
@@ -153,7 +147,7 @@ proc ::label { id type text } {
 
     # create a new label with and a box behind it
 
-    set color [lindex [$c configure -background] 4]
+    set color [$c cget -background]
 
     if {$type == "NETWORK"} {
 	set xy [$id labelxy]
@@ -202,6 +196,7 @@ proc ::postscript { id } {
 proc NODE::canvas { node } {
     $node items [[$node canvas] create bitmap \
 		  [lindex [$node move] 0] [lindex [$node move] 1] \
+		      -background [[$node canvas] cget -background] \
 		      -bitmap machine -tags [list NODE "id $node"] ]
 }
 
@@ -319,7 +314,8 @@ proc GROUP::collapse { group } {
 
     $group items [[$group canvas] create bitmap \
 		    [lindex [$group move] 0] [lindex [$group move] 1] \
-			-bitmap group -tags [list GROUP "id $group"] ]
+		      -background [[$group canvas] cget -background] \
+		      -bitmap group -tags [list GROUP "id $group"] ]
 }
 
 proc GROUP::expand { group } {
@@ -567,11 +563,22 @@ proc IMAGE::canvas { image } {
     set x [lindex [$image move] 0]
     set y [lindex [$image move] 1]
     set tags [list IMAGE "id $image"]
+if {0} {
     if {[catch {$c create bitmap $x $y -bitmap @$fname -tags $tags} item]} {
 	Dialog::acknowledge $c "Image file not readable!"
 	$image canvas ""
 	return
     }
+} else {
+    if {[catch {image create photo -file $fname} img]} {
+	if {[catch {image create bitmap -file $fname -maskfile $fname} img]} {
+	    Dialog::acknowledge $c "Image file not readable!" "" $img
+	    $image canvas ""
+	    return
+	}
+    }
+    set item [$c create image $x $y -image $img -tags $tags]
+}
     $image items $item
     $image lower
 }
@@ -628,10 +635,8 @@ proc MENU::canvas { menu } {
 		-command [list MENU::send $menu $newcmd]
 	}
     }
-    pack $name -side left
 
-    # update the traversal bindings
-    eval tk_menuBar $w.menu [pack slaves $w.menu]
+    pack $name -side left
 
     $interpreter items "$menu [$interpreter items]"
 }
@@ -652,10 +657,8 @@ proc MENU::delete { menu } {
     set interpreter [$menu interpreter]
     set c [$menu canvas]
     set w [winfo parent $c]
-    destroy $w.menu.[$menu id]
 
-    # update the traversal bindings
-    eval tk_menuBar $w.menu [pack slaves $w.menu]
+    destroy $w.menu.[$menu id]
 
     # and remove the menu from the interpreter
     set new ""
@@ -671,6 +674,9 @@ proc MENU::delete { menu } {
 
 ##
 ## The following set of procedures handle interpreter objects.
+## The delete method is a dummy that is needed in cases where
+## the creation of a new interpreter fails (e.g. we can't open
+## another pipe).
 ##
 
 proc INTERPRETER::queue { interpreter qlen } {
@@ -688,6 +694,9 @@ proc INTERPRETER::queue { interpreter qlen } {
     }
 }
 
+proc INTERPRETER::delete { interpreter } {
+}
+
 
 ##
 ## The following set of procedures handle log objects.
@@ -708,40 +717,84 @@ proc LOG::canvas { log } {
     set w [winfo parent $c]
     toplevel $w.$log
 
-    frame $w.$log.button 
-    button $w.$log.button.ok    -padx 5 -text dismiss \
-	-command "$log delete"
-    button $w.$log.button.clear -padx 5 -text clear \
+    # set up the menu bar
+    frame $w.$log.menu -borderwidth 1 -relief raised
+
+    # set up the file menu
+    menubutton $w.$log.menu.file -text "File" -menu $w.$log.menu.file.m
+    menu $w.$log.menu.file.m
+    $w.$log.menu.file.m add command -label "Clear" \
+	-accelerator "  Alt+C" \
 	-command "$log clear"
-    button $w.$log.button.save  -padx 5 -text save \
-	-command "LOG::save $log"
-    button $w.$log.button.load  -padx 5 -text load \
+    bind $w.$log <Alt-c>  "$w.$log.menu.file.m invoke Clear"
+    bind $w.$log <Alt-C>  "$w.$log.menu.file.m invoke Clear"
+    bind $w.$log <Meta-c> "$w.$log.menu.file.m invoke Clear"
+    bind $w.$log <Meta-C> "$w.$log.menu.file.m invoke Clear"
+    $w.$log.menu.file.m add command -label "Open..." \
+	-accelerator "  Alt+O" \
 	-command "LOG::load $log"
-    button $w.$log.button.print  -padx 5 -text print \
+    bind $w.$log <Alt-o>  "$w.$log.menu.file.m invoke Open..."
+    bind $w.$log <Alt-O>  "$w.$log.menu.file.m invoke Open..."
+    bind $w.$log <Meta-o> "$w.$log.menu.file.m invoke Open..."
+    bind $w.$log <Meta-O> "$w.$log.menu.file.m invoke Open..."
+    $w.$log.menu.file.m add command -label "Save As..." \
+	-accelerator "  Alt+S" \
+	-command "LOG::save $log"
+    bind $w.$log <Alt-s>  "$w.$log.menu.file.m invoke {Save As...}"
+    bind $w.$log <Alt-S>  "$w.$log.menu.file.m invoke {Save As...}"
+    bind $w.$log <Meta-s> "$w.$log.menu.file.m invoke {Save As...}"
+    bind $w.$log <Meta-S> "$w.$log.menu.file.m invoke {Save As...}"
+    $w.$log.menu.file.m add sep
+    $w.$log.menu.file.m add command -label "Print..." \
+	-accelerator "  Alt+P" \
 	-command "LOG::print $log"
-    button $w.$log.button.email -padx 5 -text email \
+    bind $w.$log <Alt-p>  "$w.$log.menu.file.m invoke Print..."
+    bind $w.$log <Alt-P>  "$w.$log.menu.file.m invoke Print..."
+    bind $w.$log <Meta-p> "$w.$log.menu.file.m invoke Print..."
+    bind $w.$log <Meta-P> "$w.$log.menu.file.m invoke Print..."
+    $w.$log.menu.file.m add command -label "Email..." \
+	-accelerator "  Alt+E" \
 	-command "LOG::email $log"
-    button $w.$log.button.up -padx 5 -text up \
-	-command "LOG::up $log"
-    button $w.$log.button.down -padx 5 -text down \
-	-command "LOG::down $log"
-    button $w.$log.button.freeze -padx 5 -text freeze \
-	-command "LOG::freeze $log"
-    pack $w.$log.button.clear  -pady 5 -padx 2 -side left
-    pack $w.$log.button.up     -pady 5 -padx 2 -side left
-    pack $w.$log.button.down   -pady 5 -padx 2 -side left
-    pack $w.$log.button.freeze -pady 5 -padx 2 -side left
-    pack $w.$log.button.load   -pady 5 -padx 2 -side left
-    pack $w.$log.button.save   -pady 5 -padx 2 -side left
-    pack $w.$log.button.print  -pady 5 -padx 2 -side left
-    pack $w.$log.button.email  -pady 5 -padx 2 -side left
-    pack $w.$log.button.ok     -pady 5 -padx 2 -side left
-    pack $w.$log.button -anchor w
+    bind $w.$log <Alt-e>  "$w.$log.menu.file.m invoke Email..."
+    bind $w.$log <Alt-E>  "$w.$log.menu.file.m invoke Email..."
+    bind $w.$log <Meta-e> "$w.$log.menu.file.m invoke Email..."
+    bind $w.$log <Meta-E> "$w.$log.menu.file.m invoke Email..."
+    $w.$log.menu.file.m add sep
+    $w.$log.menu.file.m add command -label "New View" \
+	-accelerator "  Alt+N" \
+	-command "set l \[LOG create\]; \$l canvas $c; \$l name \[$log name\]"
+    bind $w.$log <Alt-n>  "$w.$log.menu.file.m invoke {New View}"
+    bind $w.$log <Alt-N>  "$w.$log.menu.file.m invoke {New View}"
+    bind $w.$log <Meta-n> "$w.$log.menu.file.m invoke {New View}"
+    bind $w.$log <Meta-N> "$w.$log.menu.file.m invoke {New View}"
+    $w.$log.menu.file.m add command -label "Close View" \
+	-accelerator "  Alt+W" \
+	-command "$log delete"
+    bind $w.$log <Alt-w>  "$w.$log.menu.file.m invoke {Close View}"
+    bind $w.$log <Alt-W>  "$w.$log.menu.file.m invoke {Close View}"
+    bind $w.$log <Meta-w> "$w.$log.menu.file.m invoke {Close View}"
+    bind $w.$log <Meta-W> "$w.$log.menu.file.m invoke {Close View}"
+
+    # set up the option menu
+    menubutton $w.$log.menu.opts -text "Options" -menu $w.$log.menu.opts.m
+    menu $w.$log.menu.opts.m
+    $w.$log.menu.opts.m add checkbutton -label "Freeze" \
+	-offvalue 0 -onvalue 1 -variable freeze$log \
+	-accelerator "  ^Z"
+    bind $w.$log <Control-z> "$w.$log.menu.opts.m invoke Freeze"
+    $w.$log.menu.opts.m add checkbutton -label "Wrap" \
+	-offvalue none -onvalue word -variable wrap$log \
+	-accelerator "  ^W" -command "LOG::wrap $log"
+    bind $w.$log <Control-w> "$w.$log.menu.opts.m invoke Wrap"
+
+    pack $w.$log.menu -side top -fill x
+    pack $w.$log.menu.file -side left
+    pack $w.$log.menu.opts -side left
 
     scrollbar $w.$log.scrollbar -command "$w.$log.text yview" -relief sunken
-    text $w.$log.text -height 24 -width 80 -setgrid true \
+    text $w.$log.text -height 24 -width 80 -setgrid true -wrap none \
 	-relief sunken -borderwidth 2 \
-	-yscrollcommand "$w.$log.scrollbar set" 
+	-yscrollcommand "$w.$log.scrollbar set"
     pack $w.$log.scrollbar -side right -fill y
     pack $w.$log.text -side left -padx 2 -pady 2 -fill both -expand yes
 
@@ -753,10 +806,6 @@ proc LOG::canvas { log } {
     # command frontend.
 
     bind $w.$log.text <Shift-Return> "LOG::process $log"
-    bind $w.$log.text <Control-n>    "LOG::clear $log"
-    bind $w.$log.text <Control-o>    "LOG::load $log"
-    bind $w.$log.text <Control-s>    "LOG::save $log"
-    bind $w.$log.text <Control-q>    "$log delete"
 
     # Position the log window on the screen.
 
@@ -777,7 +826,7 @@ proc LOG::canvas { log } {
     if {$y < 0} { set y 0 }
 
     wm geometry $w.$log +$x+$y
-    wm deiconify $w.$log    
+    wm deiconify $w.$log
 
     update idletasks
 }
@@ -785,9 +834,10 @@ proc LOG::canvas { log } {
 proc LOG::process { log } {
 
     set w [$log items].text
-    set i1 [$w index "insert linestart"]
-    set i2 [$w index "insert"]
-    set txt [$w get $i1 $i2]
+    $w delete insert
+#    set line [$w index "insert - 1 line"]
+    set line insert
+    set txt [$w get [$w index "$line linestart"] [$w index "$line lineend"]]
     if {$txt == ""} return
 
     [$log interpreter] send \
@@ -806,9 +856,10 @@ proc LOG::icon { log bitmap } {
 }
 
 proc LOG::append { log text } {
+    global freeze$log
     set w [$log items]
     $w.text insert end $text
-    if {[lindex [$w.button.freeze configure -text] 4] != "melt"} {
+    if {! [set freeze$log]} {
 	$w.text yview -pickplace end
     }
 }
@@ -817,28 +868,33 @@ proc LOG::bind { log cmd text } {
 
     set w [$log items].text
 
-    if {[tk colormodel $w] == "color"} {
-        set bold "-foreground red"
-        set normal "-foreground {}"
-    } else {
-        set bold "-foreground white -background black"
-        set normal "-foreground {} -background {}"
-    }
+if {1} {
+    set bold "-foreground red"
+    set normal "-foreground {}"
 
-    $w insert end "<"
-    set start [$w index end]
-    $w insert $start $text
-    $w yview -pickplace end
-    set end [$w index end]
-    $w insert end ">"
+    set start [$w index insert]
+    $w insert $start "<$text>"
+    set end [$w index insert]
     set tag "tag$start$end"
     $w tag add $tag $start $end
+    $w yview -pickplace end
 
-    $w tag configure $tag -underline true
+#    $w tag configure $tag -borderwidth 1 -relief groove
+    $w tag configure $tag -underline 1
     $w tag bind $tag <Any-Enter> "$w tag configure $tag $bold"
     $w tag bind $tag <Any-Leave> "$w tag configure $tag $normal"
     $w tag bind $tag <Button-3> "[$log interpreter] send $cmd"
     $w tag bind $tag <Button-1> "[$log interpreter] send $cmd"
+} else {
+
+    static idx
+    if {![info exists idx]} {set idx 0}
+    incr idx
+    button $w.$idx -text "<$text>" -font [$w cget -font] \
+	-command "[$log interpreter] send $cmd" \
+	-padx 0 -pady 0 -relief groove
+    $w window create end -window $w.$idx
+}
 }
 
 proc LOG::unbind { log } {
@@ -856,22 +912,19 @@ proc LOG::delete { log } {
     destroy [$log items]
 }
 
-proc LOG::up { log } {
-    [$log items].text yview "@0,0 - 24 lines"
-}
-
-proc LOG::down { log } {
-    [$log items].text yview "@0,0 + 24 lines"
-}
-
 proc LOG::freeze { log } {
-    if {[lindex [[$log items].button.freeze configure -text] 4] == "freeze"} {
-	[$log items].button.freeze configure -text melt
+    if {[[$log items].menu.freeze cget -text] == "freeze"} {
+	[$log items].menu.freeze configure -text melt
     } else {
-	[$log items].button.freeze configure -text freeze
+	[$log items].menu.freeze configure -text freeze
     }
 }
 
+proc LOG::wrap { log } {
+    global wrap$log
+    [$log items].text configure -wrap [set wrap$log]
+}
+
 proc LOG::save { log } {
 
     set fname [Dialog::fileselect [$log items] "Write to file:"]
@@ -901,22 +954,29 @@ proc LOG::save { log } {
 
     puts $file [[$log items].text get 1.0 end]
     close $file
+
+    $log attribute filename $fname
 }
 
 proc LOG::load { log } {
-    set fname [Dialog::fileselect [$log items] "Read from file:"]
-    if {$fname==""} return
+    set dir [file dirname [$log attribute filename]]
+    set fname [Dialog::fileselect [$log items] "Read from file:" $dir]
+    if {$fname == ""} return
 
     if {[catch {open $fname r} file]} {
 	Dialog::acknowledge [$log items] "Unable to read from $fname"
 	return
     }
 
-    $log clear
+    $log attribute filename $fname
+
+    set txt ""
     while {![eof $file]} {
-	gets $file line
-	[$log items].text insert end "$line\n"
+	append txt "[gets $file]\n"
     }
+    $log clear
+    $log append $txt
+
     [$log items].text yview -pickplace end
     [$log items].text yview 1.0
     close $file
@@ -943,7 +1003,7 @@ proc LOG::print { log } {
 
     catch {close $file}
 
-    tkined_print [$log editor] [$log items] $fname
+    Editor::print [$log editor] [$log items] $fname
 
     catch {exec /bin/rm -f $fname}
 }
@@ -1000,19 +1060,25 @@ proc REFERENCE::canvas { ref } {
     set c [$ref canvas]
     $ref items [$c create bitmap \
 		[lindex [$ref move] 0] [lindex [$ref move] 1] \
-		    -bitmap reference -tags [list REFERENCE "id $ref"] ]
+		      -background [[$ref canvas] cget -background] \
+		      -bitmap reference -tags [list REFERENCE "id $ref"] ]
+}
+
+proc REFERENCE::open { reference {editor ""} } {
+    if {$editor == ""} {
+	set editor [$reference editor]
+    }
+    foreach w [winfo children .] { $w configure -cursor watch }
+    update idletasks
+    set result [Command::Open $editor [$reference address]]
+    foreach w [winfo children .] { $w configure -cursor top_left_arrow }
+    return $result
 }
 
 proc REFERENCE::load { reference } {
-    set name [$reference address]
-    if {$name != ""} {
-	set editor [EDITOR]
-	foreach w [winfo children .] { $w configure -cursor watch }
-	update idletasks
-	if {[Command::Open $editor $name] == ""} {
-	    $editor delete
-	}
-	foreach w [winfo children .] { $w configure -cursor top_left_arrow }
+    set editor [EDITOR]
+    if {[REFERENCE::open $reference $editor] == ""} {
+	$editor delete
     }
 }
 
@@ -1071,11 +1137,9 @@ proc REFERENCE::label { reference text } {
 
 proc STRIPCHART::canvas { stripchart } {
     set c [$stripchart canvas]
-    set color [option get $c background tkined]
-    if {$color == ""} { set color white }
-    set item [$c create stripchart -31 -21 31 21 \
-	       -outline black -background $color \
-	       -tags [list STRIPCHART "id $stripchart"]]
+    set item [$c create stripchart -31 -21 31 21 -outline black \
+	    -background [$c cget -background] \
+	    -tags [list STRIPCHART "id $stripchart"]]
     eval $c move $item [$stripchart move]
     $stripchart items $item
 }
@@ -1147,7 +1211,7 @@ proc STRIPCHART::values { stripchart args } {
     set c [$stripchart canvas]
     set item [$stripchart items]
     if {$args == ""} {
-	return [lindex [$c itemconfigure $item -values] 4]
+	return [$c itemcget $item -values]
     } else {
 	$c itemconfigure $item -values [join $args]
     }
@@ -1172,15 +1236,10 @@ proc STRIPCHART::jump { stripchart num } {
 
 proc BARCHART::canvas { barchart } {
     set c [$barchart canvas]
-    set color [option get $c background tkined]
-    if {$color == ""} { set color white }
-    if {[tk colormodel .] == "color"} {
-	set item [$c create barchart -31 -21 31 21 -autocolor on \
-		   -background $color -tags [list BARCHART "id $barchart"]]
-    } else {
-	set item [$c create barchart -31 -21 31 21 -autocolor off \
-		   -background $color -tags [list BARCHART "id $barchart"]]
-    }
+    set item [$c create barchart -31 -21 31 21 \
+	    -autocolor [expr [winfo cells .] > 2] \
+	    -background [$c cget -background] \
+	    -tags [list BARCHART "id $barchart"]]
     eval $c move $item [$barchart move]
     $barchart items $item
 }
@@ -1243,7 +1302,7 @@ proc BARCHART::values { barchart args } {
     set c [$barchart canvas]
     set item [$barchart items]
     if {$args == ""} {
-	return [lindex [$c itemconfigure $item -values] 4]
+	return [$c itemcget $item -values]
     } else {
 	$c itemconfigure $item -values [join $args]
     }
@@ -1290,11 +1349,13 @@ proc GRAPH::icon { graph dashes } {
 }
 
 proc GRAPH::clearlabel { graph } {
-    [$graph canvas] element configure $graph -label ""
+## XXX got memory errors
+##    [$graph canvas] element configure $graph -label ""
 }
 
 proc GRAPH::label { graph text } {
-    [$graph canvas] element configure $graph -label $text
+## XXX got memory errors
+##    [$graph canvas] element configure $graph -label $text
 }
 
 proc GRAPH::clear { graph } {
@@ -1313,3 +1374,19 @@ proc GRAPH::postscript { graph } {
     $c postscript -fontmap tkined_ps_map	    
 }
 
+##
+##
+##
+
+proc DATA::canvas { data } {
+    set s [STRIPCHART create]
+    $s canvas [$data canvas]
+    $data items $s
+}
+
+proc DATA::values { data args } {
+    set values [join $args]
+    foreach proc [info commands STREAM:*] {
+	eval [list $proc $data $values]
+    }
+}
