@@ -1,12 +1,12 @@
 ##
 ## Tool.tcl
 ##
-## This file contains the tools of the tkined editor. A Tool is something
+## This file contains the tools of the Tkined editor. A Tool is something
 ## you can select and apply on the canvas. You will normally have an
-## animation bind to you mouse. This is the most complicated stuff and 
+## animation bind to mouse events. This is the most complicated stuff and 
 ## needs some more clean ups.
 ##
-## Copyright (c) 1993, 1994
+## Copyright (c) 1993, 1994, 1995
 ##
 ## J. Schoenwaelder
 ## TU Braunschweig, Germany
@@ -24,23 +24,30 @@
 
 ##
 ## Find the topmost item at position x and y that is of one of 
-## the tkined types given in the list types.
+## the Tkined types given in the list types.
 ##
 
 proc Tool::Find { c x y types } {
+    set lowx [expr $x-5]
+    set lowy [expr $y-5]
+    set upx  [expr $x+5]
+    set upy  [expr $y+5]
     set first [$c find closest $x $y 3]
     set item $first
     while 1 {
-	set tags [$c gettags $item]
-	if {[lsearch -glob $tags label*] >= 0 && [lsearch $types LABEL] >= 0} {
-	    return $item
-	}
-	foreach type $types {
-	    if {[lsearch $tags $type] >= 0} {
+	set inregion [$c find overlapping $lowx $lowy $upx $upy]
+	if {[lsearch $inregion $item] >= 0} {
+	    set tags [$c gettags $item]
+	    if {[lsearch -glob $tags label*]>=0 && [lsearch $types LABEL]>=0} {
 		return $item
 	    }
+	    foreach type $types {
+		if {[lsearch $tags $type] >= 0} {
+		    return $item
+		}
+	    }
 	}
-
+	    
 	set item [$c find closest $x $y 3 $item]
 	if {$item == $first} return ""
     }
@@ -64,35 +71,57 @@ proc Tool::GetId { c item } {
 ## Make a popup menu for the alter tool. 
 ##
 
-proc tkined_makepopup { popup id } {
+proc Tool::MakePopup { popup id } {
 
-    $popup add command -label "Create Attribute" -command \
-	"tkined_createatt $id"
+    $popup add command -label "Create a new Attribute..." \
+	    -command "Tool::CreateAttribute $id"
     if {[$id attribute] != ""} {
-	$popup add command -label "Delete Attribute" -command \
-	    "tkined_deleteatt $id"
+	$popup add command -label "Delete an Attribute..." \
+		-command "Tool::DeleteAttribute $id"
     }
-    $popup add command -label "Edit Attributes" -command \
-	"tkined_editatt $id"
+    if {[$id type] != "LINK" || [$id attribute] != ""} {
+	$popup add command -label "Edit all Attributes..." \
+		-command "Tool::EditAttribute $id"
+    }
+    if {[$id type] != "LINK"} {
+	$popup add command -label "Label with Attribute..." \
+		-command "Tool::LabelAttribute $id"
+    }
 
-    $popup add separator
-
-    $popup add command -label "Label: name" \
-	-command "$id label name; destroy $popup"
-    $popup add command -label "Label: address" \
-	-command "$id label address; destroy $popup"
-
-    foreach att [$id attribute] {
-	if {[string match "_*" $att]} continue
-	$popup add command -label "Label: $att" \
-	    -command "[list $id label $att] ; destroy $popup"
+    set editor [$id editor]
+    set w [$editor toplevel]
+    set cnt 0
+    foreach menu [$editor retrieve MENU] {
+	set attr [$id attribute "Tkined:[$menu name]"]
+	if {$attr == ""} {
+	    set attr [$editor attribute [$menu name]]
+	}
+	if {$attr == ""} continue
+	set inter [$menu interpreter]
+	set items ""
+	foreach item [$menu items] {
+	    lappend items [lindex [split $item :] end]
+	}
+	set args ""
+	foreach cmd $attr {
+	    if {[lsearch $items $cmd] < 0} continue
+	    lappend args $cmd
+	}
+	if {$args == ""} continue
+	if {[incr cnt] == 1} {
+	    $popup add separator
+	}
+	$popup add cascade -label [$menu name] -menu $popup.$menu \
+		-state [$w.menu.$menu cget -state]
+	menu $popup.$menu -tearoff false
+	foreach cmd $args {
+	    $popup.$menu add command -label $cmd -command \
+		    [list [$menu interpreter] send $cmd [list [$id retrieve]]]
+	}
     }
-    
-    $popup add command -label "Clear Label" -command \
-	"$id label clear; focus [winfo parent [$id canvas]]; destroy $popup"
 }
 
-proc tkined_createatt { id } {
+proc Tool::CreateAttribute { id } {
 
     set result [Dialog::request [$id canvas] "Create an attribute for $id:" \
 		[list [list Name: "" ] [list Value: "" ] ] \
@@ -103,25 +132,29 @@ proc tkined_createatt { id } {
     $id attribute [lindex $result 1] [lindex $result 2]
 }
 
-proc tkined_deleteatt { id } {
+proc Tool::DeleteAttribute { id } {
 
     set result [Dialog::list [$id canvas] "Delete an attribute of $id:" \
-		[$id attribute] [list delete cancel] ]
+		[lsort [$id attribute]] [list delete cancel] ]
 
     if {[lindex $result 0] == "cancel"} return
 
     $id attribute [lindex $result 1] ""
 }
 
-proc tkined_editatt { id } {
+proc Tool::EditAttribute { id } {
 
-    lappend list [list name [$id name]]
-    if {[$id type] != "GROUP"} {
-	lappend list [list address [$id address]]
+    set list ""
+    if {[$id type] != "LINK"} {
+	lappend list [list name [$id name] entry 40]
+	if {[$id type] != "GROUP"} {
+	    lappend list [list address [$id address] entry 40]
+	}
     }
-    foreach att [$id attribute] {
-	lappend list [list $att [$id attribute $att]]
+    foreach att [lsort [$id attribute]] {
+	lappend list [list $att [$id attribute $att] entry 40]
     }
+    if {$list == ""} return
 
     set result [Dialog::request [$id canvas] "Edit attributes on $id:" \
 		$list [list "set values" cancel] ]
@@ -142,6 +175,27 @@ proc tkined_editatt { id } {
     }
 }
 
+proc Tool::LabelAttribute { id } {
+    set list [concat "name address" [lsort [$id attribute]]]
+    set result [Dialog::list [$id canvas] \
+	    "Select an attribute as a label for $id:" $list \
+	    [list accept "no label" cancel] ]
+    switch [lindex $result 0] {
+	cancel {
+	    return
+	}
+	"no label" {
+	    $id label clear
+	}
+	default {
+	    set attribute [lindex $result 1]
+	    if {$attribute != ""} {
+		$id label $attribute
+	    }
+	}
+    }
+}
+
 ##
 ## This procedure should be called whenever a tool is selected.
 ## It sets the relief of the button to sunken after resetting the
@@ -153,6 +207,8 @@ proc Tool::Init { editor tool } {
     set w [$editor toplevel]
     set c $w.canvas
 
+    $c configure -cursor top_left_arrow
+
     $c delete area
     $c delete network
     $c delete line
@@ -174,6 +230,8 @@ proc Tool::Init { editor tool } {
     bind $c <Shift-Button-1> ""
     bind $c <Motion> ""
     bind $c <Shift-Motion> ""
+    bind $c <Shift-B1-Motion> ""
+    bind $c <Shift-ButtonRelease-1> ""
 
     # these bindings allow drag operations on the canvas
     bind $w.canvas <Shift-2> "Tool::ScrollMark $w.canvas %x %y"
@@ -211,11 +269,21 @@ proc Tool::Select { editor } {
     
     bind $c <1> \
 	"$editor selection clear; \
-         Tool::SelectApply $editor \
-         \[$c canvasx %x\] \[$c canvasy %y\]"
+         Tool::SelectMark $editor \
+         \[%W canvasx %x\] \[%W canvasy %y\]"
+    bind $c <B1-Motion> \
+        "Tool::SelectDrag $editor \
+            \[%W canvasx %x\] \[%W canvasy %y\]"
+    bind $c <ButtonRelease-1> \
+        "Tool::SelectDone $editor"
     bind $c <Shift-Button-1> \
-	"Tool::SelectApply $editor \
-	\[$c canvasx %x\] \[$c canvasy %y\]"
+	"Tool::SelectMark $editor \
+	    \[%W canvasx %x\] \[%W canvasy %y\]"
+    bind $c <Shift-B1-Motion> \
+        "Tool::SelectDrag $editor \
+            \[%W canvasx %x\] \[%W canvasy %y\]"
+    bind $c <Shift-ButtonRelease-1> \
+        "Tool::SelectDone $editor"
 }
 
 proc Tool::SelectApply { editor x y } {
@@ -250,7 +318,7 @@ proc Tool::SelectMark { editor x y } {
 	if {[lsearch $tags IMAGE]<0} break
 	set tkined_valid 1
     }
-    if {$tkined_valid} {
+     if {$tkined_valid} {
 	set tkined_areaX1 $x
 	set tkined_areaY1 $y
 	set tkined_areaX2 $x
@@ -265,11 +333,11 @@ proc Tool::SelectMark { editor x y } {
 proc Tool::SelectDrag { editor x y } {
     global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2 \
 	tkined_cursor tkined_valid
-    if {!$tkined_valid} return
+    if {![info exists tkined_valid] || !$tkined_valid} return
     set c [$editor toplevel].canvas    
     if {($tkined_areaX1 != $x) && ($tkined_areaY1 != $y)} {
 	$c delete area
-	$c create rect $tkined_areaX1 $tkined_areaY1 \
+	$c create rectangle $tkined_areaX1 $tkined_areaY1 \
 	    $x $y -outline black -tags "area"
 	set tkined_areaX2 $x
 	set tkined_areaY2 $y
@@ -287,7 +355,7 @@ proc Tool::SelectDrag { editor x y } {
 	    set new_cursor top_left_corner
 	}
     }
-    if {$new_cursor!="$tkined_cursor"} {
+    if {$new_cursor!=$tkined_cursor} {
 	set tkined_cursor $new_cursor
 	$c configure -cursor $tkined_cursor
     }
@@ -296,7 +364,7 @@ proc Tool::SelectDrag { editor x y } {
 proc Tool::SelectDone { editor } {
     global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2 \
 	tkined_valid
-    if {!$tkined_valid} return
+    if {![info exists tkined_valid] || !$tkined_valid} return
     set w [$editor toplevel]
     set c $w.canvas    
     $w.tools.select configure -state disabled
@@ -308,11 +376,9 @@ proc Tool::SelectDone { editor } {
         if {$id == ""} continue
 	set table($id) ""
     }
-    if {[info exist table]} {
-	foreach id [array names table] {
-	    if {[catch {$id selected} sel]} continue
-	    if {$sel} { $id unselect } else { $id select }
-	}
+    foreach id [array names table] {
+	if {[catch {$id selected} sel]} continue
+	if {$sel} { $id unselect } else { $id select }
     }
     $c delete area
     $c configure -cursor top_left_arrow
@@ -324,17 +390,13 @@ proc Tool::SelectDone { editor } {
 ##
 
 proc Tool::Move { editor } {
-
-    Tool::Init $editor move
-
-    set c [$editor toplevel].canvas
-    
-    bind $c <1> "Tool::MoveMark $editor \
-                \[$c canvasx %x\] \[$c canvasy %y\]"
-    bind $c <B1-Motion> "Tool::MoveDrag $editor \
-                \[$c canvasx %x\] \[$c canvasy %y\]"
-    bind $c <ButtonRelease-1> "Tool::MoveDone $editor \
-                \[$c canvasx %x\] \[$c canvasy %y\]"
+    set w [$editor toplevel]
+    bind $w.canvas <2> "Tool::MoveMark $editor \
+                \[$w.canvas canvasx %x\] \[$w.canvas canvasy %y\]"
+    bind $w.canvas <B2-Motion> "Tool::MoveDrag $editor \
+                \[$w.canvas canvasx %x\] \[$w.canvas canvasy %y\]"
+    bind $w.canvas <ButtonRelease-2> "Tool::MoveDone $editor \
+                \[$w.canvas canvasx %x\] \[$w.canvas canvasy %y\]"
 }
 
 proc Tool::MoveMark { editor x y } {
@@ -346,16 +408,30 @@ proc Tool::MoveMark { editor x y } {
     set y1 [expr {$y-5}]
     set x2 [expr {$x+5}]
     set y2 [expr {$y+5}]
+    set unselected ""
     set tkined_valid 0
     foreach item [$c find overlapping $x1 $y1 $x2 $y2] {
 	set id [Tool::GetId $c $item]
 	if {$id == ""} continue
 	if {[catch {$id selected} sel]} continue
-	if {[set tkined_valid $sel]} {
+        if {$sel} {
+	    set tkined_valid 1
 	    break
+	} else {
+	    set unselected $id
 	}
     }
-    if {!$tkined_valid} return
+    if {! $tkined_valid && $unselected != ""} {
+	$editor selection clear
+	if {[catch {$unselected select}]} return
+	set tkined_valid 1
+    }
+    if {!$tkined_valid} {
+	Tool::ScrollMark $c [expr int($x)] [expr int($y)]
+	bind $c <B2-Motion> "Tool::ScrollDrag $c %x %y"
+	bind $c <ButtonRelease-2> "Tool::ScrollDone $c; Tool::Move $editor"
+	return
+    }
     set tkined_startX $x
     set tkined_startY $y
     set tkined_oldX $x
@@ -370,7 +446,7 @@ proc Tool::MoveMark { editor x y } {
     }
     if {$tkined_valid} {
 	set area [eval $c bbox $selection]
-	eval $c create rect $area -outline black -tags "area"
+	eval $c create rectangle $area -outline black -tags "area"
     }
 }
 
@@ -394,8 +470,7 @@ proc Tool::MoveDone { editor x y } {
     set undo_cmd ""
     if {!$tkined_valid} return
     set w [$editor toplevel]
-    set c $w.canvas    
-    $w.tools.move configure -state disabled
+    set c $w.canvas
     update idletask
     set dx [expr $x-$tkined_startX]
     set dy [expr $y-$tkined_startY]
@@ -425,7 +500,6 @@ proc Tool::MoveDone { editor x y } {
 	Command::Undo $editor [list $undo_cmd $redo_cmd]
     }
     $c delete area
-    $w.tools.move configure -state normal
 }
 
 ##
@@ -439,6 +513,8 @@ proc Tool::Text { editor } {
     Tool::Init $editor text
 
     set c [$editor toplevel].canvas
+
+    $c configure -cursor xterm
     
     bind $c <1> "Tool::TextMark $editor \
                 \[$c canvasx %x\] \[$c canvasy %y\]"
@@ -462,13 +538,22 @@ proc Tool::TextMark { editor x y } {
     if {$x11colorname == ""} { set x11colorname black }  
     set item [$c create text $x $y -anchor nw -tags "text" \
 	      -font $x11fontname -fill $x11colorname]
-    $c bind $item <KeyPress>       "$c insert $item insert %A"
-    $c bind $item <Shift-KeyPress> "$c insert $item insert %A"
-    $c bind $item <Return>         "$c insert $item insert \\n"
-    $c bind $item <BackSpace>      "Tool::TextBs $c $item"
-    $c bind $item <Control-h>      "Tool::TextBs $c $item"
-    $c bind $item <Delete>         "Tool::TextBs $c $item"
-    bind $c <FocusOut>             "Tool::TextDone $editor"
+    $c bind $item <KeyPress>       "$c insert $item insert %A;  break"
+    $c bind $item <Shift-KeyPress> "$c insert $item insert %A;  break"
+    $c bind $item <Return>         "$c insert $item insert \\n; break"
+    $c bind $item <BackSpace>      "Tool::TextBs $c $item; break"
+    $c bind $item <Control-h>      "Tool::TextBs $c $item; break"
+    $c bind $item <Delete>         "Tool::TextBs $c $item; break"
+    # make sure that no toplevel bindings are invoked
+    bind $c <KeyPress> "break"
+
+    $c bind $item <Any-Leave>      "Tool::TextDone $editor"
+##    bind $c <FocusOut>             "Tool::TextDone $editor"
+
+    # XXX a tk4.0 hack - perhaps it will go when tk4.0 gets stable ...
+    set x [expr int($x)]
+    set y [expr int($y)]
+
     $c icursor $item @$x,$y
     $c focus $item
     focus $c
@@ -480,7 +565,7 @@ proc Tool::TextDone { editor } {
     set c $w.canvas
     foreach item [$c find withtag text] {
 	set xy [$c coords $item]
-	set txt [lindex [$c itemconfigure $item -text] 4]
+	set txt [$c itemcget $item -text]
 	set text [TEXT create $txt]
 	$text editor $editor
 	$text canvas $c
@@ -494,29 +579,27 @@ proc Tool::TextDone { editor } {
 }
 
 ##
-## The Label tool can be used to get and set informations about
-## an object. The various attributes are shown in the label of
-## the object.
+## The Resize tool is used to resize strip- or barcharts and to
+## stretch or shrink networks.
 ##
 
-proc Tool::Alter { editor } {
-
-    Tool::Init $editor alter
+proc Tool::Resize { editor } {
+    Tool::Init $editor resize
 
     set c [$editor toplevel].canvas
-    
-    bind $c <1> "Tool::AlterMark $editor \
+
+    bind $c <1> "Tool::ResizeMark $editor \
                 \[$c canvasx %x\] \[$c canvasy %y\] %X %Y"
 }
 
-proc Tool::AlterMark { editor cx cy xx xy } {
+proc Tool::ResizeMark { editor cx cy xx xy } {
 
     set c [$editor toplevel].canvas
     $c delete scale
 
-    set item [Tool::Find $c $cx $cy \
-	      "NODE GROUP NETWORK TEXT REFERENCE STRIPCHART BARCHART LABEL"]
+    set item [Tool::Find $c $cx $cy "STRIPCHART BARCHART NETWORK"]
     if {$item == ""} return
+
     set x1 [expr {$cx-5}]
     set y1 [expr {$cy-5}]
     set x2 [expr {$cx+5}]
@@ -527,14 +610,8 @@ proc Tool::AlterMark { editor cx cy xx xy } {
 	set type LABEL
     }
     switch $type {
-	STRIPCHART {
-	    Tool::AlterChartMenu $editor $id $item $xx $xy $x1 $y1 $x2 $y2
-	}
-	BARCHART {
-	    Tool::AlterChartMenu $editor $id $item $xx $xy $x1 $y1 $x2 $y2
-	}
-	NODE  { Tool::AlterNode $editor $id $item $xx $xy }
-	NETWORK { 
+
+	NETWORK {
 
 	    set bb [$c coords $item]
 	    set len [llength $bb]
@@ -545,58 +622,195 @@ proc Tool::AlterMark { editor cx cy xx xy } {
 	    set by2 [lindex $bb $len]
 
 	    if { ($x1<$bx1) && ($bx1<$x2) && ($y1<$by1) && ($by1<$y2)} {
-		Tool::AlterNetworkLen $editor $id $item $bx2 $by2 $bx1 $by1
+		Tool::ResizeNetworkMark $editor $id $item $bx2 $by2 $bx1 $by1
 		return
 	    }
 	    
 	    if { ($x1<$bx2) && ($bx2<$x2) && ($y1<$by2) && ($by2<$y2)} {
-		Tool::AlterNetworkLen $editor $id $item $bx1 $by1 $bx2 $by2
+		Tool::ResizeNetworkMark $editor $id $item $bx1 $by1 $bx2 $by2
 		return
 	    }
-		    
-	    Tool::AlterNetwork $editor $id $item $xx $xy 
 	}
+	STRIPCHART -
+	BARCHART {
+
+	    set bb [$c bbox $item]
+	    set bx1 [lindex $bb 0]
+	    set by1 [lindex $bb 1]
+	    set bx2 [lindex $bb 2]
+	    set by2 [lindex $bb 3]
+    
+	    if { ($x1<$bx1) && ($bx1<$x2) && ($y1<$by1) && ($by1<$y2)} {
+		Tool::ResizeChart $editor $id $item $bx2 $by2 $bx1 $by1
+		return
+	    }
+	    
+	    if { ($x1<$bx1) && ($bx1<$x2) && ($y1<$by2) && ($by2<$y2)} {
+		Tool::ResizeChart $editor $id $item $bx2 $by1 $bx1 $by2
+		return
+	    }
+	    
+	    if { ($x1<$bx2) && ($bx2<$x2) && ($y1<$by1) && ($by1<$y2)} {
+		Tool::ResizeChart $editor $id $item $bx1 $by2 $bx2 $by1
+		return
+	    }
+	    
+	    if { ($x1<$bx2) && ($bx2<$x2) && ($y1<$by2) && ($by2<$y2)} {
+		Tool::ResizeChart $editor $id $item $bx1 $by1 $bx2 $by2
+		return
+	    }
+	}
+    }
+		    
+}
+
+proc Tool::ResizeNetworkMark { editor id item x1 y1 x2 y2 } {
+    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+    set c [$editor toplevel].canvas
+    set tkined_areaX1 $x1
+    set tkined_areaY1 $y1
+    set tkined_areaX2 $x2
+    set tkined_areaY2 $y2
+
+    $c delete area
+
+    if {$x1 == $x2} {
+	bind $c <B1-Motion> \
+	    "Tool::ResizeNetworkDrag $editor $id $item $x1 \[$c canvasy %y\]"
+	bind $c <ButtonRelease-1> \
+	    "Tool::ResizeNetworkDone $editor $id $item $x1 \[$c canvasy %y\]"
+    }
+
+    if {$y1 == $y2} {
+	bind $c <B1-Motion> \
+	    "Tool::ResizeNetworkDrag $editor $id $item \[$c canvasx %x\] $y1"
+	bind $c <ButtonRelease-1> \
+	    "Tool::ResizeNetworkDone $editor $id $item \[$c canvasx %x\] $y1"
+    }
+}
+
+proc Tool::ResizeNetworkDrag { editor id item x y } {
+    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+    set c [$editor toplevel].canvas
+
+    set tkined_areaX2 $x
+    set tkined_areaY2 $y
+    $c coords $item $tkined_areaX1 $tkined_areaY1 $tkined_areaX2 $tkined_areaY2
+}
+
+proc Tool::ResizeNetworkDone { editor id item x y } {
+    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+    set c [$editor toplevel].canvas
+
+    set sx [lindex [$id move] 0]
+    set sy [lindex [$id move] 1]
+    if {$sx == $tkined_areaX1 && $sy == $tkined_areaY1} {
+	set tkined_areaX2 [expr $tkined_areaX2 - $tkined_areaX1]
+	set tkined_areaY2 [expr $tkined_areaY2 - $tkined_areaY1]
+	$id points "0 0 $tkined_areaX2 $tkined_areaY2"
+    } else {
+	set dx [expr {$tkined_areaX2 - $sx}]
+	set dy [expr {$tkined_areaY2 - $sy}]
+	$c move $item [expr {-1 * $dx}] [expr {-1 * $dy}]
+	$id move $dx $dy
+	set tkined_areaX2 [expr $tkined_areaX1 - $tkined_areaX2]
+	set tkined_areaY2 [expr $tkined_areaY1 - $tkined_areaY2]
+	$id points "0 0 $tkined_areaX2 $tkined_areaY2"
+    }
+    Tool::Resize $editor
+}
+
+proc Tool::ResizeChart { editor id item x1 y1 x2 y2 } {
+    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+    set c [$editor toplevel].canvas
+    set tkined_areaX1 $x1
+    set tkined_areaY1 $y1
+    set tkined_areaX2 $x2
+    set tkined_areaY2 $y2
+    $c delete area
+    bind $c <B1-Motion> \
+	"Tool::ResizeChartDrag $editor $id $item \
+            \[$c canvasx %x\] \[$c canvasy %y\]"
+    bind $c <ButtonRelease-1> \
+	"Tool::ResizeChartDone $editor $id $item \
+            \[$c canvasx %x\] \[$c canvasy %y\]"
+}
+
+proc Tool::ResizeChartDrag { editor id item x y } {
+    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+    set c [$editor toplevel].canvas
+    if {($tkined_areaX1 != $x) && ($tkined_areaY1 != $y)} {
+	set tkined_areaX2 $x
+	set tkined_areaY2 $y
+	$c delete area
+	$c create rectangle \
+	    $tkined_areaX1 $tkined_areaY1 $tkined_areaX2 $tkined_areaY2 \
+	    -outline black -tags "area"
+    }
+}
+
+proc Tool::ResizeChartDone { editor id item x y } {
+    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+    set c [$editor toplevel].canvas
+    $c delete area
+    $id size $tkined_areaX1 $tkined_areaY1 $tkined_areaX2 $tkined_areaY2
+    Tool::Resize $editor
+}
+
+##
+## The Alter tool can be used to get and set informations about
+## an object. The various attributes are shown in the label of
+## the object.
+##
+
+proc Tool::Alter { editor } {
+
+    Tool::Init $editor alter
+
+    set c [$editor toplevel].canvas
+
+    bind $c <1> "Tool::AlterMark $editor \
+                \[$c canvasx %x\] \[$c canvasy %y\] %X %Y"
+}
+
+proc Tool::AlterMark { editor cx cy xx xy } {
+
+    set c [$editor toplevel].canvas
+    $c delete scale
+
+    # XXX a tk4.0 hack - perhaps it will go when tk4.0 gets stable ...
+    set cx [expr int($cx)]
+    set cy [expr int($cy)]
+
+    set item [Tool::Find $c $cx $cy \
+	      "NODE GROUP NETWORK TEXT REFERENCE STRIPCHART BARCHART LINK"]
+    if {$item == ""} return
+    set x1 [expr $cx - 5]
+    set y1 [expr $cy - 5]
+    set x2 [expr $cx + 5]
+    set y2 [expr $cy + 5]
+    set id [Tool::GetId $c $item]
+    set type [$id type]
+    switch $type {
+	STRIPCHART -
+	BARCHART {
+	    Tool::AlterChart $editor $id $item $xx $xy $x1 $y1 $x2 $y2
+	}
+	NODE      { Tool::AlterNode      $editor $id $item $xx $xy }
+	NETWORK   { Tool::AlterNetwork   $editor $id $item $xx $xy }
 	GROUP     { Tool::AlterGroup     $editor $id $item $xx $xy }
 	REFERENCE { Tool::AlterReference $editor $id $item $xx $xy }
-	LABEL     { Tool::AlterLabel     $editor $id $item $cx $cy }
+	LINK	  { Tool::AlterLink	 $editor $id $item $xx $xy }
 	TEXT      { Tool::AlterText      $editor $id $item $cx $cy }
     }
 }
 
-proc Tool::AlterChartMenu { editor id item x y x1 y1 x2 y2 } {
-
+proc Tool::AlterChart { editor id item x y x1 y1 x2 y2 } {
     set c [$editor toplevel].canvas
-
-    set bb [$c bbox $item]
-    set bx1 [lindex $bb 0]
-    set by1 [lindex $bb 1]
-    set bx2 [lindex $bb 2]
-    set by2 [lindex $bb 3]
-    
-    if { ($x1<$bx1) && ($bx1<$x2) && ($y1<$by1) && ($by1<$y2)} {
-	Tool::AlterChart $editor $id $item $bx2 $by2 $bx1 $by1
-	return
-    }
-    
-    if { ($x1<$bx1) && ($bx1<$x2) && ($y1<$by2) && ($by2<$y2)} {
-	Tool::AlterChart $editor $id $item $bx2 $by1 $bx1 $by2
-	return
-    }
-    
-    if { ($x1<$bx2) && ($bx2<$x2) && ($y1<$by1) && ($by1<$y2)} {
-	Tool::AlterChart $editor $id $item $bx1 $by2 $bx2 $by1
-	return
-    }
-
-    if { ($x1<$bx2) && ($bx2<$x2) && ($y1<$by2) && ($by2<$y2)} {
-	Tool::AlterChart $editor $id $item $bx1 $by1 $bx2 $by2
-	return
-    }
-
     catch {destroy $c.popup}
-    menu $c.popup
+    menu $c.popup -tearoff false
 
-    tkined_makepopup $c.popup $id 
+    Tool::MakePopup $c.popup $id 
 
     set chart [string tolower [$id type]]
     $c.popup add separator
@@ -611,20 +825,7 @@ proc Tool::AlterChartMenu { editor id item x y x1 y1 x2 y2 } {
     $c.popup add command -label "Clear $chart" \
 	-command "$id clear; destroy $c.popup"
 
-    $c.popup post [incr x -20] [incr y -50]
-    bind $c.popup <Any-Leave> {destroy %W}
-}
-
-proc Tool::AlterNode { editor id item x y } {
-
-    set c [$editor toplevel].canvas
-    catch {destroy $c.popup}
-    menu $c.popup
-
-    tkined_makepopup $c.popup $id 
-
-    $c.popup post [incr x -20] [incr y -50]
-    bind $c.popup <Any-Leave> {destroy %W}
+    tk_popup $c.popup $x $y
 }
 
 proc Tool::AlterChartScale { editor id item x y } {
@@ -638,7 +839,7 @@ proc Tool::AlterChartScale { editor id item x y } {
     set py [lindex $bb 3]
 
     set max 100
-    foreach val [lindex [$c itemconfigure $item -values] 4] {
+    foreach val [$c itemcget $item -values] {
 	set max [expr {($max < $val) ? int(ceil($val/100)*100) : $max}]
     }
     set old [$id scale]
@@ -665,7 +866,7 @@ proc Tool::AlterChartJump { editor id item x y } {
 
     set max [$c bbox $item]
     set max [expr {[lindex $max 2] - [lindex $max 0] - 6}]
-    set val [lindex [$c itemconfigure $item -jump] 4]
+    set val [$c itemcget $item -jump]
 
     scale $c.scale -borderwidth 1 -sliderlength 5 -font fixed \
 	-from 1 -to $max \
@@ -676,183 +877,83 @@ proc Tool::AlterChartJump { editor id item x y } {
 	-tags "scale" -anchor se -height $height
 }
 
-proc Tool::AlterChart { editor id item x1 y1 x2 y2 } {
-    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+proc Tool::AlterNode { editor id item x y } {
     set c [$editor toplevel].canvas
-    set tkined_areaX1 $x1
-    set tkined_areaY1 $y1
-    set tkined_areaX2 $x2
-    set tkined_areaY2 $y2
-    $c delete area
-    bind $c <B1-Motion> \
-	"Tool::AlterChartDrag $editor $id $item \
-            \[$c canvasx %x\] \[$c canvasy %y\]"
-    bind $c <ButtonRelease-1> \
-	"Tool::AlterChartDone $editor $id $item \
-            \[$c canvasx %x\] \[$c canvasy %y\]"
+    catch {destroy $c.popup}
+    menu $c.popup -tearoff false
+
+    Tool::MakePopup $c.popup $id
+
+    tk_popup $c.popup $x $y
 }
 
-proc Tool::AlterChartDrag { editor id item x y } {
-    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
+proc Tool::AlterLink { editor id item x y } {
     set c [$editor toplevel].canvas
-    if {($tkined_areaX1 != $x) && ($tkined_areaY1 != $y)} {
-	set tkined_areaX2 $x
-	set tkined_areaY2 $y
-	$c delete area
-	$c create rect \
-	    $tkined_areaX1 $tkined_areaY1 $tkined_areaX2 $tkined_areaY2 \
-	    -outline black -tags "area"
-    }
-}
+    catch {destroy $c.popup}
+    menu $c.popup -tearoff false
 
-proc Tool::AlterChartDone { editor id item x y } {
-    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
-    set c [$editor toplevel].canvas
-    $c delete area
-    $id size $tkined_areaX1 $tkined_areaY1 $tkined_areaX2 $tkined_areaY2
-    Tool::Alter $editor
+    Tool::MakePopup $c.popup $id
+
+    tk_popup $c.popup $x $y
 }
 
 proc Tool::AlterGroup { editor id item x y } {
     set c [$editor toplevel].canvas
     catch {destroy $c.popup}
-    menu $c.popup
+    menu $c.popup -tearoff false
 
-    tkined_makepopup $c.popup $id 
+    Tool::MakePopup $c.popup $id 
 
-    $c.popup post [incr x -20] [incr y -50]
-    bind $c.popup <Any-Leave> {destroy %W}
+    $c.popup add separator
+    if {[$id collapsed]} {
+	$c.popup add command -label "Expand group" \
+	    -command "destroy $c.popup; $id expand"
+    } else {
+	$c.popup add command -label "Collapse group" \
+	    -command "destroy $c.popup; $id collapse"
+    }
+
+    tk_popup $c.popup $x $y
 }
 
 proc Tool::AlterReference { editor id item x y } {
     set c [$editor toplevel].canvas
     catch {destroy $c.popup}
-    menu $c.popup
+    menu $c.popup -tearoff false
 
-    tkined_makepopup $c.popup $id 
+    Tool::MakePopup $c.popup $id 
 
     $c.popup add separator
-    $c.popup add command -label "Follow reference" \
+    $c.popup add command -label "Open (this view)" \
+	-command "destroy $c.popup; REFERENCE::open $id"
+    $c.popup add command -label "Open (new view)" \
 	-command "destroy $c.popup; REFERENCE::load $id"
 
-    $c.popup post [incr x -20] [incr y -50]
-    bind $c.popup <Any-Leave> {destroy %W}
-}
-
-proc Tool::AlterNetworkLen { editor id item x1 y1 x2 y2 } {
-    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
-    set c [$editor toplevel].canvas
-    set tkined_areaX1 $x1
-    set tkined_areaY1 $y1
-    set tkined_areaX2 $x2
-    set tkined_areaY2 $y2
-
-    $c delete area
-
-    if {$x1 == $x2} {
-	bind $c <B1-Motion> \
-	    "Tool::AlterNetworkDrag $editor $id $item $x1 \[$c canvasy %y\]"
-	bind $c <ButtonRelease-1> \
-	    "Tool::AlterNetworkDone $editor $id $item $x1 \[$c canvasy %y\]"
-    }
-
-    if {$y1 == $y2} {
-	bind $c <B1-Motion> \
-	    "Tool::AlterNetworkDrag $editor $id $item \[$c canvasx %x\] $y1"
-	bind $c <ButtonRelease-1> \
-	    "Tool::AlterNetworkDone $editor $id $item \[$c canvasx %x\] $y1"
-    }
-}
-
-proc Tool::AlterNetworkDrag { editor id item x y } {
-    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
-    set c [$editor toplevel].canvas
-
-    set tkined_areaX2 $x
-    set tkined_areaY2 $y
-    $c coords $item $tkined_areaX1 $tkined_areaY1 $tkined_areaX2 $tkined_areaY2
-}
-
-proc Tool::AlterNetworkDone { editor id item x y } {
-    global tkined_areaX1 tkined_areaY1 tkined_areaX2 tkined_areaY2
-    set c [$editor toplevel].canvas
-
-    set sx [lindex [$id move] 0]
-    set sy [lindex [$id move] 1]
-    if {$sx == $tkined_areaX1 && $sy == $tkined_areaY1} {
-	set tkined_areaX2 [expr $tkined_areaX2 - $tkined_areaX1]
-	set tkined_areaY2 [expr $tkined_areaY2 - $tkined_areaY1]
-	$id points "0 0 $tkined_areaX2 $tkined_areaY2"
-    } else {
-	set dx [expr {$tkined_areaX2 - $sx}]
-	set dy [expr {$tkined_areaY2 - $sy}]
-	$c move $item [expr {-1 * $dx}] [expr {-1 * $dy}]
-	$id move $dx $dy
-	set tkined_areaX2 [expr $tkined_areaX1 - $tkined_areaX2]
-	set tkined_areaY2 [expr $tkined_areaY1 - $tkined_areaY2]
-	$id points "0 0 $tkined_areaX2 $tkined_areaY2"
-    }
-    Tool::Alter $editor
+    tk_popup $c.popup $x $y
 }
 
 proc Tool::AlterNetwork { editor id item x y } {
     set c [$editor toplevel].canvas
     catch {destroy $c.popup}
-    menu $c.popup
+    menu $c.popup -tearoff false
 
-    tkined_makepopup $c.popup $id 
+    Tool::MakePopup $c.popup $id 
 
-    $c.popup post [incr x -20] [incr y -50]
-    bind $c.popup <Any-Leave> {destroy %W}
-}
-
-proc Tool::AlterLabel { editor id item x y } {
-    set c [$editor toplevel].canvas
-    $c bind $item <Return>         "focus [winfo toplevel $c]"
-    $c bind $item <KeyPress>       "$c insert $item insert %A"
-    $c bind $item <Shift-KeyPress> "$c insert $item insert %A"
-    $c bind $item <BackSpace>      "Tool::TextBs $c $item"
-    $c bind $item <Control-h>      "Tool::TextBs $c $item"
-    $c bind $item <Control-a>      "$c icursor $item 0"
-    $c bind $item <Control-e>      "$c icursor $item end"
-    $c bind $item <Delete>         "Tool::TextBs $c $item"
-    $c bind $item <2> "$c insert $item insert \[selection get\]"
-    # save the new label when we leave the current focus
-    bind $c <FocusOut>             "Tool::AlterLabelDone $editor $id $item"
-    $c bind $item <Any-Leave>      "Tool::AlterLabelDone $editor $id $item"
-    $c icursor $item @$x,$y
-    $c focus $item
-    focus $c
-}
-
-proc Tool::AlterLabelDone { editor id item } {
-    set c [$editor toplevel].canvas
-    $c bind $item <2> ""
-    set txt [lindex [$c itemconfigure $item -text] 4]
-    switch [$id label] {
-	name {
-	    $id name $txt
-	}
-	address {
-	    $id address $txt
-	}
-	default {
-	    $id attribute [$id label] $txt
-	}
-    }
+    tk_popup $c.popup $x $y
 }
 
 proc Tool::AlterText { editor id item x y } {
     set c [$editor toplevel].canvas
-    $c bind $item <Return>         "$c insert $item insert \\n"
-    $c bind $item <KeyPress>       "$c insert $item insert %A"
-    $c bind $item <Shift-KeyPress> "$c insert $item insert %A"
-    $c bind $item <BackSpace>      "Tool::TextBs $c $item"
-    $c bind $item <Control-h>      "Tool::TextBs $c $item"
-    $c bind $item <Control-a>      "$c icursor $item 0"
-    $c bind $item <Control-e>      "$c icursor $item end"
-    $c bind $item <Delete>         "Tool::TextBs $c $item"
-    $c bind $item <2> "$c insert $item insert \[selection get\]"
+    $c bind $item <Return>         "$c insert $item insert \\n; break"
+    $c bind $item <KeyPress>       "$c insert $item insert %A;  break"
+    $c bind $item <Shift-KeyPress> "$c insert $item insert %A;  break"
+    $c bind $item <BackSpace>      "Tool::TextBs $c $item; break"
+    $c bind $item <Control-h>      "Tool::TextBs $c $item; break"
+    $c bind $item <Control-a>      "$c icursor $item 0; break"
+    $c bind $item <Control-e>      "$c icursor $item end;  break"
+    $c bind $item <Delete>         "Tool::TextBs $c $item; break"
+    # make sure that no toplevel bindings are invoked
+    bind $c <KeyPress> "break"
     # save the new text when we leave the current focus
     bind $c <FocusOut>             "Tool::AlterTextDone $editor $id $item"
     $c icursor $item @$x,$y
@@ -862,9 +963,12 @@ proc Tool::AlterText { editor id item x y } {
 
 proc Tool::AlterTextDone { editor id item } {
     set c [$editor toplevel].canvas
-    $c bind $item <2> ""
-    set txt [lindex [$c itemconfigure $item -text] 4]
-    $id text $txt
+    bind $c <FocusOut> ""
+    catch {
+	$c bind $item <2> ""
+	set txt [$c itemcget $item -text]
+	$id text $txt
+    }
 }
 
 ##
@@ -878,6 +982,8 @@ proc Tool::Node { editor } {
 
     set c [$editor toplevel].canvas
     
+    $c configure -cursor plus
+
     bind $c <1> "Tool::NodeDone $editor \[$c canvasx %x\] \[$c canvasy %y\]"
 }
 
@@ -906,6 +1012,8 @@ proc Tool::Network { editor } {
 
     set c [$editor toplevel].canvas
     
+    $c configure -cursor plus
+    
     bind $c <1> "Tool::NetworkMark $editor \
                 \[$c canvasx %x\] \[$c canvasy %y\]"
 }
@@ -1001,6 +1109,8 @@ proc Tool::Link { editor } {
 
     set c [$editor toplevel].canvas
     
+    $c configure -cursor plus
+    
     bind $c <1> "Tool::LinkMark $editor \
                 \[$c canvasx %x\] \[$c canvasy %y\]"
 }
@@ -1115,6 +1225,8 @@ proc Tool::Group { editor } {
 
     set c [$editor toplevel].canvas
 
+    $c configure -cursor plus
+
     bind $c <1> "Tool::GroupDone $editor \[$c canvasx %x\] \[$c canvasy %y\]"
 }
 
@@ -1143,6 +1255,8 @@ proc Tool::Reference { editor } {
 
     set c [$editor toplevel].canvas
 
+    $c configure -cursor plus
+
     bind $c <1> \
 	"Tool::ReferenceDone $editor \[$c canvasx %x\] \[$c canvasy %y\]"
 }
