@@ -3,7 +3,7 @@
  *
  * Some utility functions that are used in other places.
  *
- * Copyright (c) 1993, 1994
+ * Copyright (c) 1993, 1994, 1995
  *
  * J. Schoenwaelder
  * TU Braunschweig, Germany
@@ -18,20 +18,10 @@
  * express or implied warranty.
  */
 
-#include <stdlib.h>
-#include <malloc.h>
-#include <string.h>
-#include <pwd.h>
-#include <errno.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include "tkined.h"
 
 /*
- * A general purpose sprintf buffer. See the tkined.h 
- * for its intended use.
+ * A general purpose sprintf buffer. See tkined.h for its intended use.
  */
 
 char *buffer;
@@ -50,74 +40,17 @@ buffersize (size)
 
     if (size < 1024) size = 1024;
 
-    if (buffer_size == 0) 
-	    buffer = xmalloc(size);
-    else 
-	    if (size > buffer_size) 
-		    buffer = xrealloc (buffer, size);
+    if (buffer_size == 0) {
+	buffer = ckalloc(size);
+    } else {
+	if (size > buffer_size) {
+	    buffer = ckrealloc (buffer, size);
+	}
+    }
 
     buffer_size = size;
 }
 
-#ifndef DBMALLOC
-
-/*
- * A fail-save malloc.
- */
-
-char*
-xmalloc (size)
-    unsigned size;
-{
-    char *mem;
-
-    for (;;) 
-    {
-	mem = (char*) malloc(size);
-	if (mem) break;
-	sleep (1);
-    }
-    
-    return mem;
-}
-
-/*
- * A fail-save realloc.
- */
-
-char*
-xrealloc (ptr, size)
-    char *ptr;
-    unsigned size;
-{
-    char *mem;
-    
-    if (!ptr) 
-	    mem = (char*) xmalloc(size);
-    else
-	    for (;;) 
-	    {
-		mem = (char*) realloc(ptr, size);
-		if (mem) break;
-		sleep (1);
-	    }
-    
-    return mem;
-}
-
-/*
- * Define out own strdup. At least our ultrix machine does not have
- * it. We generally use xstrudp and xmalloc.
- */
-
-char*
-xstrdup (s) 
-    char *s;
-{
-    return strcpy(xmalloc(strlen(s)+1), s);
-}
-#endif
-
 /*
  * Write a buffer to a file descriptor. This wrapper is needed on
  * broken SYS V machines.
@@ -175,7 +108,10 @@ findfile (name)
 
     buffersize (strlen(name)+10);
 
-    /* substitue ~ - we do it by hand to avoid the need for an interpreter */
+    /*
+     * Substitue ~ - we do it by hand to avoid the need for an 
+     * interpreter handle.
+     */
 
     if (name[0] == '~') {
 
@@ -220,7 +156,7 @@ findfile (name)
 
     if ((tmp = getenv("TKINED_PATH")) != NULL) {
 	char *p, *s;
-	tmp = xstrdup(tmp);
+	tmp = ckstrdup(tmp);
 	for (s = p = tmp; *p; p++) {
 	    if (*p == ':') {
 		*p = '\0';
@@ -228,7 +164,7 @@ findfile (name)
 		strcat (buffer, "/");
 		strcat (buffer, name);
 		if (access(buffer, R_OK) == 0) {
-		    free (tmp);
+		    ckfree (tmp);
 		    return buffer;
 		}
 		s = ++p;
@@ -239,11 +175,11 @@ findfile (name)
 	    strcat (buffer, "/");
 	    strcat (buffer, name);
 	    if (access(buffer, R_OK) == 0) {
-		free (tmp);
+		ckfree (tmp);
 		return buffer;
 	    }
 	}
-	free (tmp);
+	ckfree (tmp);
     }
 
     if ((tmp = getenv("HOME")) != NULL) {
@@ -303,6 +239,9 @@ type_to_string (type)
 	case TKINED_STRIPCHART:  result = "STRIPCHART"; break;
 	case TKINED_BARCHART:    result = "BARCHART"; break;
 	case TKINED_GRAPH:	 result = "GRAPH"; break;
+	case TKINED_HTML:	 result = "HTML"; break;
+	case TKINED_DATA:	 result = "DATA"; break;
+	case TKINED_EVENT:	 result = "EVENT"; break;
 	default:                 result = "";
     }
 
@@ -347,6 +286,12 @@ string_to_type (str)
 	else if ((*str == 'G') && (strcmp(str, "GRAPH") == 0))
 		type = TKINED_STRIPCHART;
 #endif
+	else if ((*str == 'H') && (strcmp(str, "HTML") == 0))
+		type = TKINED_HTML;
+	else if ((*str == 'D') && (strcmp(str, "DATA") == 0))
+		type = TKINED_DATA;
+	else if ((*str == 'E') && (strcmp(str, "EVENT") == 0))
+		type = TKINED_EVENT;
    }
 
     return type;
@@ -379,7 +324,7 @@ ldelete (interp, slist, item)
 	    strcat (slist, " ");
         }
     }
-    free ((char*) largv);
+    ckfree ((char*) largv);
 
     i = strlen (slist) - 1;
     if (slist[i] == ' ') slist[i] = '\0';
@@ -394,7 +339,7 @@ lappend (slist, item)
     char **slist;
     char *item;
 {
-    *slist = xrealloc (*slist, strlen (*slist) + strlen(item) + 2);
+    *slist = ckrealloc (*slist, strlen (*slist) + strlen(item) + 2);
     if (**slist != '\0') strcat (*slist, " ");
     strcat (*slist, item);
 }
@@ -406,7 +351,7 @@ lappend (slist, item)
  */
 
 char*
-xstrdupnn (s) 
+ckstrdupnn (s)
     char *s;
 {
     char *p, *t, *r;
@@ -417,7 +362,7 @@ xstrdupnn (s)
     }
 
     n += (p - s);
-    r = xmalloc (n);
+    r = ckalloc (n);
 
     for (t = r, p = s; *p != 0; p++, t++) {
 	if (*p == '\n') {
