@@ -5,9 +5,11 @@
  *      It's based on tkRectOval.c and tkCanvPoly.c from the 
  *      tk distribution by John Ousterhout.
  *
- * Copyright (c) 1993-1994 G. Hueske (hueske@ibr.cs.tu-bs.de)
- *               TU Braunschweig, Germany
- *               Institute for Operating System and Computer Networks
+ * Copyright (c) 1993, 1994, 1995
+ *
+ * G. Hueske (hueske@ibr.cs.tu-bs.de)
+ * TU Braunschweig, Germany
+ * Institute for Operating System and Computer Networks
  *
  * Permission to use, copy, modify, and distribute this
  * software and its documentation for any purpose and without
@@ -19,9 +21,14 @@
  */
 
 #include <stdio.h>
-#include <tkInt.h>
-#include <tkCanvas.h>
-#include <tkConfig.h>
+#include <stdlib.h>
+#include <math.h>
+#include <ctype.h>
+#include <tk.h>
+
+#ifndef UCHAR
+#define UCHAR(c) ((unsigned char) (c))
+#endif
 
 /*
  * The structure below defines the record for each stripchart item.
@@ -30,8 +37,9 @@
 typedef struct StripchartItem {
     Tk_Item header;             /* Generic stuff that's the same for all
                                  * types.  MUST BE FIRST IN STRUCTURE. */
-    Tk_Canvas *canvasPtr;       /* Canvas containing item. Needed for
+    Tk_Canvas canvas;           /* Canvas containing item. Needed for
 				 * parsing stripchart values. */
+    Tcl_Interp *interp;		/* Interpreter handle. */
     int numPoints;              /* Number of points in stripchart polygon. */
     double *coordPtr;           /* Pointer to malloc-ed array containing
                                  * x- and y-coords of all points in polygon.
@@ -41,8 +49,6 @@ typedef struct StripchartItem {
 				 * the y-coords of all points in polygon. */
     int scale;                  /* Actual scale of the stripchart. */
     double scaleValue;          /* Every scalevalue we scale up. */
-    int scaledUp;               /* Boolean indicator whether the program
-				 * decided to increase the scalevalue. */
     int jump;                   /* Number of pixels jumped with a 
 				 * jumpscroll. */
     int numValues;              /* Number of values that are actually 
@@ -75,17 +81,19 @@ typedef struct StripchartItem {
  */
 
 static void             ComputeStripchartBbox _ANSI_ARGS_((
-                            Tk_Canvas *canvasPtr, StripchartItem *stripPtr));
-static int              ConfigureStripchart _ANSI_ARGS_((
-                            Tk_Canvas *canvasPtr, Tk_Item *itemPtr, int argc,
+                            Tk_Canvas canvas, StripchartItem *stripPtr));
+static int              ConfigureStripchart _ANSI_ARGS_((Tcl_Interp *interp,
+                            Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
                             char **argv, int flags));
-static int              CreateStripchart _ANSI_ARGS_((Tk_Canvas *canvasPtr,
-                            struct Tk_Item *itemPtr, int argc, char **argv));
-static void             DeleteStripchart _ANSI_ARGS_((Tk_Canvas *canvasPtr,
-                            Tk_Item *itemPtr));
-static void             DisplayStripchart _ANSI_ARGS_((Tk_Canvas *canvasPtr,
-                            Tk_Item *itemPtr, Drawable dst));
-static void             FillStripchart _ANSI_ARGS_((Tk_Canvas *canvasPtr, 
+static int              CreateStripchart _ANSI_ARGS_((Tcl_Interp *interp,
+			    Tk_Canvas canvas, Tk_Item *itemPtr,
+			    int argc, char **argv));
+static void             DeleteStripchart _ANSI_ARGS_((Tk_Canvas canvas,
+                            Tk_Item *itemPtr, Display *display));
+static void             DisplayStripchart _ANSI_ARGS_((Tk_Canvas canvas,
+                            Tk_Item *itemPtr, Display *display, Drawable dst,
+			    int x, int y, int width, int height));
+static void             FillStripchart _ANSI_ARGS_((Tk_Canvas canvas, 
 			    Tk_Item *itemPtr, double *coordPtr, 
 			    Drawable drawable));
 static int              ParseStripchartValues _ANSI_ARGS_((ClientData 
@@ -94,21 +102,22 @@ static int              ParseStripchartValues _ANSI_ARGS_((ClientData
 static char *           PrintStripchartValues _ANSI_ARGS_((ClientData 
 			    clientData, Tk_Window tkwin, char *recordPtr, 
 			    int offset, Tcl_FreeProc **freeProcPtr));
-static void             ScaleStripchart _ANSI_ARGS_((Tk_Canvas *canvasPtr,
+static void             ScaleStripchart _ANSI_ARGS_((Tk_Canvas canvas,
                             Tk_Item *itemPtr, double originX, double originY,
                             double scaleX, double scaleY));
-static int              StripchartCoords _ANSI_ARGS_((Tk_Canvas *canvasPtr,
-                            Tk_Item *itemPtr, int argc, char **argv));
-static int              StripchartToArea _ANSI_ARGS_((Tk_Canvas *canvasPtr,
+static int              StripchartCoords _ANSI_ARGS_((Tcl_Interp *interp,
+			    Tk_Canvas canvas, Tk_Item *itemPtr, 
+                            int argc, char **argv));
+static int              StripchartToArea _ANSI_ARGS_((Tk_Canvas canvas,
                             Tk_Item *itemPtr, double *rectPtr));
-static double           StripchartToPoint _ANSI_ARGS_((Tk_Canvas *canvasPtr,
+static double           StripchartToPoint _ANSI_ARGS_((Tk_Canvas canvas,
                             Tk_Item *itemPtr, double *pointPtr));
-static int              StripchartToPostscript _ANSI_ARGS_((
-                            Tk_Canvas *canvasPtr, Tk_Item *itemPtr, 
-                            Tk_PostscriptInfo *psInfoPtr));
-static int              StripchartValues _ANSI_ARGS_ ((Tk_Canvas *canvasPtr, 
-			    Tk_Item *itemPtr, int argc, char **argv));
-static void             TranslateStripchart _ANSI_ARGS_((Tk_Canvas *canvasPtr,
+static int              StripchartToPostscript _ANSI_ARGS_((Tcl_Interp *interp,
+                            Tk_Canvas canvas, Tk_Item *itemPtr, int prepass));
+static int              StripchartValues _ANSI_ARGS_ ((Tcl_Interp *interp,
+			    Tk_Canvas canvas, Tk_Item *itemPtr, 
+                            int argc, char **argv));
+static void             TranslateStripchart _ANSI_ARGS_((Tk_Canvas canvas,
                             Tk_Item *itemPtr, double deltaX, double deltaY));
 
 /*
@@ -139,7 +148,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_COLOR, "-stripline", (char *) NULL, (char *) NULL,
         "", Tk_Offset(StripchartItem, striplineColor), TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-tags", (char *) NULL, (char *) NULL,
-        (char *) NULL, 0, TK_CONFIG_NULL_OK, &tkCanvasTagsOption},
+        (char *) NULL, 0, TK_CONFIG_NULL_OK, &tk_CanvasTagsOption},
     {TK_CONFIG_COLOR, "-text", (char *) NULL, (char *) NULL,
         "black", Tk_Offset(StripchartItem, textColor), TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-values", (char *) NULL, (char *) NULL,
@@ -208,8 +217,9 @@ Tk_ItemType TkStripchartType = {
  */
 
 static int
-CreateStripchart(canvasPtr, itemPtr, argc, argv)
-    register Tk_Canvas *canvasPtr;	/* Canvas to hold new item. */
+CreateStripchart(interp, canvas, itemPtr, argc, argv)
+    Tcl_Interp *interp;			/* Interpreter for error reporting. */
+    Tk_Canvas canvas;			/* Canvas to hold new item. */
     Tk_Item *itemPtr;			/* Record to hold new item;  header
 					 * has been initialized by caller. */
     int argc;				/* Number of arguments in argv. */
@@ -219,8 +229,8 @@ CreateStripchart(canvasPtr, itemPtr, argc, argv)
     int i;
     
     if (argc < 4) {
-	Tcl_AppendResult(canvasPtr->interp, "wrong # args:  should be \"",
-		Tk_PathName(canvasPtr->tkwin), "\" create ",
+	Tcl_AppendResult(interp, "wrong # args:  should be \"",
+		Tk_PathName(Tk_CanvasTkwin(canvas)), "\" create ",
 		itemPtr->typePtr->name,
 		" x1 y1 x2 y2 ?options?", (char *) NULL);
 	return TCL_ERROR;
@@ -230,13 +240,14 @@ CreateStripchart(canvasPtr, itemPtr, argc, argv)
      * Carry out initialization that is needed in order to clean
      * up after errors during the the remainder of this procedure.
      */
-    stripPtr->canvasPtr = canvasPtr;
+
+    stripPtr->canvas = canvas;
+    stripPtr->interp = interp;
     stripPtr->numPoints = 0;
     stripPtr->coordPtr = NULL;
     stripPtr->shiftPtr = NULL;
     stripPtr->scale = 1;
     stripPtr->scaleValue = 100.0;
-    stripPtr->scaledUp = 0;
     stripPtr->jump = 5;
     stripPtr->numValues = 0;
     stripPtr->stripColor = NULL;
@@ -266,16 +277,17 @@ CreateStripchart(canvasPtr, itemPtr, argc, argv)
 	}
     }
 
-    if (StripchartCoords(canvasPtr, itemPtr, i, argv) != TCL_OK) {
+    if (StripchartCoords(interp, canvas, itemPtr, i, argv) != TCL_OK) {
 	goto error;
     }
 
-    if (ConfigureStripchart(canvasPtr, itemPtr, argc-i, argv+i, 0) == TCL_OK) {
+    if (ConfigureStripchart(interp, canvas, itemPtr, argc-i, argv+i, 0)
+	   == TCL_OK) {
 	return TCL_OK;
     }
 
     error:
-    DeleteStripchart(canvasPtr, itemPtr);
+    DeleteStripchart(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
     return TCL_ERROR;
 }
 
@@ -298,8 +310,9 @@ CreateStripchart(canvasPtr, itemPtr, argc, argv)
  */
 
 static int
-StripchartCoords(canvasPtr, itemPtr, argc, argv)
-    register Tk_Canvas *canvasPtr;	/* Canvas containing item. */
+StripchartCoords(interp, canvas, itemPtr, argc, argv)
+    Tcl_Interp *interp;			/* Used for error reporting. */
+    Tk_Canvas canvas;			/* Canvas containing item. */
     Tk_Item *itemPtr;			/* Item whose coordinates are to be
 					 * read or modified. */
     int argc;				/* Number of coordinates supplied in
@@ -312,33 +325,33 @@ StripchartCoords(canvasPtr, itemPtr, argc, argv)
     char c2[TCL_DOUBLE_SPACE], c3[TCL_DOUBLE_SPACE];
 
     if (argc == 0) {
-	Tcl_PrintDouble(canvasPtr->interp, stripPtr->bbox[0], c0);
-        Tcl_PrintDouble(canvasPtr->interp, stripPtr->bbox[1], c1);
-        Tcl_PrintDouble(canvasPtr->interp, stripPtr->bbox[2], c2);
-        Tcl_PrintDouble(canvasPtr->interp, stripPtr->bbox[3], c3);
-        Tcl_AppendResult(canvasPtr->interp, c0, " ", c1, " ",
+	Tcl_PrintDouble(interp, stripPtr->bbox[0], c0);
+        Tcl_PrintDouble(interp, stripPtr->bbox[1], c1);
+        Tcl_PrintDouble(interp, stripPtr->bbox[2], c2);
+        Tcl_PrintDouble(interp, stripPtr->bbox[3], c3);
+        Tcl_AppendResult(interp, c0, " ", c1, " ",
                 c2, " ", c3, (char *) NULL);
     } else if (argc != 4) {
-	Tcl_AppendResult(canvasPtr->interp, "wrong # args:  should be \"",
-		Tk_PathName(canvasPtr->tkwin), 
+	Tcl_AppendResult(interp, "wrong # args:  should be \"",
+		Tk_PathName(Tk_CanvasTkwin(canvas)), 
 		"\" coords tagOrId x1 y1 x2 y2", (char *) NULL);
 	return TCL_ERROR;
     }  else {
-	if ((TkGetCanvasCoord(canvasPtr, argv[0],
+	if ((Tk_CanvasGetCoord(interp, canvas, argv[0],
 		    &stripPtr->bbox[0]) != TCL_OK)
-		|| (TkGetCanvasCoord(canvasPtr, argv[1],
+		|| (Tk_CanvasGetCoord(interp, canvas, argv[1],
 		    &stripPtr->bbox[1]) != TCL_OK)
-		|| (TkGetCanvasCoord(canvasPtr, argv[2],
+		|| (Tk_CanvasGetCoord(interp, canvas, argv[2],
 			&stripPtr->bbox[2]) != TCL_OK)
-		|| (TkGetCanvasCoord(canvasPtr, argv[3],
+		|| (Tk_CanvasGetCoord(interp, canvas, argv[3],
 			&stripPtr->bbox[3]) != TCL_OK)) {
 	    return TCL_ERROR;
 	}
 
     }
 
-    ComputeStripchartBbox(canvasPtr, stripPtr);   
-    StripchartValues(canvasPtr, itemPtr, 0, (char **) NULL);
+    ComputeStripchartBbox(canvas, stripPtr);   
+    StripchartValues(interp, canvas, itemPtr, 0, (char **) NULL);
     return TCL_OK;
 }
 
@@ -362,8 +375,9 @@ StripchartCoords(canvasPtr, itemPtr, argc, argv)
  */
 
 static int
-ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
-    Tk_Canvas *canvasPtr;       /* Canvas containing itemPtr. */
+ConfigureStripchart(interp, canvas, itemPtr, argc, argv, flags)
+    Tcl_Interp *interp;         /* Interpreter for error reporting. */
+    Tk_Canvas canvas;           /* Canvas containing itemPtr. */
     Tk_Item *itemPtr;           /* Stripchart item to reconfigure. */
     int argc;                   /* Number of elements in argv.  */
     char **argv;                /* Arguments describing things to configure. */
@@ -373,15 +387,17 @@ ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
     XGCValues gcValues;
     GC newGC;
     unsigned long mask;
+    Tk_Window tkwin = Tk_CanvasTkwin(canvas);
+    Display *display = Tk_Display(Tk_CanvasTkwin(canvas));
 
-    if (Tk_ConfigureWidget(canvasPtr->interp, canvasPtr->tkwin,
-	    configSpecs, argc, argv, (char *) stripPtr, flags) != TCL_OK) {
+    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, argv, 
+	    (char *) stripPtr, flags) != TCL_OK) {
 	return TCL_ERROR;
     }
 
     if (stripPtr->scaleValue <= 0) {
 	stripPtr->scaleValue = 100;
-	Tcl_AppendResult(canvasPtr->interp, 
+	Tcl_AppendResult(interp, 
 			 "wrong scalevalue: should be positiv", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -396,10 +412,10 @@ ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
     } else {
 	gcValues.foreground = stripPtr->stripColor->pixel;
 	mask = GCForeground;
-	newGC = Tk_GetGC(canvasPtr->tkwin, mask, &gcValues);
+	newGC = Tk_GetGC(tkwin, mask, &gcValues);
     }
     if (stripPtr->fillGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->fillGC);
+	Tk_FreeGC(display, stripPtr->fillGC);
     }
     stripPtr->fillGC = newGC;
 
@@ -408,10 +424,10 @@ ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
     } else {
 	gcValues.foreground = stripPtr->striplineColor->pixel;
 	mask = GCForeground;
-	newGC = Tk_GetGC(canvasPtr->tkwin, mask, &gcValues);
+	newGC = Tk_GetGC(Tk_CanvasTkwin(canvas), mask, &gcValues);
     }
     if (stripPtr->striplineGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->striplineGC);
+	Tk_FreeGC(display, stripPtr->striplineGC);
     }
     stripPtr->striplineGC = newGC;
 
@@ -421,10 +437,10 @@ ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
 	gcValues.foreground = stripPtr->outlineColor->pixel;
 	gcValues.cap_style = CapProjecting;
 	mask = GCForeground|GCCapStyle;
-	newGC = Tk_GetGC(canvasPtr->tkwin, mask, &gcValues);
+	newGC = Tk_GetGC(Tk_CanvasTkwin(canvas), mask, &gcValues);
     }
     if (stripPtr->outlineGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->outlineGC);
+	Tk_FreeGC(display, stripPtr->outlineGC);
     }
     stripPtr->outlineGC = newGC;
     
@@ -433,10 +449,10 @@ ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
     } else {
 	gcValues.foreground = stripPtr->rectColor->pixel;
 	mask = GCForeground;
-	newGC = Tk_GetGC(canvasPtr->tkwin, mask, &gcValues);
+	newGC = Tk_GetGC(Tk_CanvasTkwin(canvas), mask, &gcValues);
     }
     if (stripPtr->rectGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->rectGC);
+	Tk_FreeGC(display, stripPtr->rectGC);
     }
     stripPtr->rectGC = newGC;
 
@@ -455,15 +471,15 @@ ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
 	    gcValues.dashes = (char) stripPtr->scalelineStyle;
 	    mask |= GCLineStyle|GCDashList;
 	}
-	newGC = Tk_GetGC(canvasPtr->tkwin, mask, &gcValues);
+	newGC = Tk_GetGC(Tk_CanvasTkwin(canvas), mask, &gcValues);
     }
     if (stripPtr->scalelineGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->scalelineGC);
+	Tk_FreeGC(display, stripPtr->scalelineGC);
     }
     stripPtr->scalelineGC = newGC;
 
-    ComputeStripchartBbox(canvasPtr, stripPtr);
-    StripchartValues(canvasPtr, itemPtr, 0, (char **) NULL);
+    ComputeStripchartBbox(canvas, stripPtr);
+    StripchartValues(interp, canvas, itemPtr, 0, (char **) NULL);
     return TCL_OK;
 }
 
@@ -485,9 +501,11 @@ ConfigureStripchart(canvasPtr, itemPtr, argc, argv, flags)
  */
 
 static void
-DeleteStripchart(canvasPtr, itemPtr)
-    Tk_Canvas *canvasPtr;		/* Info about overall canvas widget. */
+DeleteStripchart(canvas, itemPtr, display)
+    Tk_Canvas canvas;			/* Info about overall canvas widget. */
     Tk_Item *itemPtr;			/* Item that is being deleted. */
+    Display *display;			/* Display containing window for
+                                         * canvas. */
 {
     register StripchartItem *stripPtr = (StripchartItem *) itemPtr;
 
@@ -501,37 +519,37 @@ DeleteStripchart(canvasPtr, itemPtr)
 	Tk_FreeColor(stripPtr->stripColor);
     }
     if (stripPtr->fillGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->fillGC);
+	Tk_FreeGC(display, stripPtr->fillGC);
     }
     if (stripPtr->striplineColor != NULL) {
 	Tk_FreeColor(stripPtr->striplineColor);
     }
     if (stripPtr->striplineGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->striplineGC);
+	Tk_FreeGC(display, stripPtr->striplineGC);
     }
     if (stripPtr->rectColor != NULL) {
 	Tk_FreeColor(stripPtr->rectColor);
     }
     if (stripPtr->rectGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->rectGC);
+	Tk_FreeGC(display, stripPtr->rectGC);
     }
     if (stripPtr->outlineColor != NULL) {
 	Tk_FreeColor(stripPtr->outlineColor);
     }
     if (stripPtr->outlineGC != None) {
-        Tk_FreeGC(canvasPtr->display, stripPtr->outlineGC);
+        Tk_FreeGC(display, stripPtr->outlineGC);
     } 
     if (stripPtr->scalelineColor != NULL) {
 	Tk_FreeColor(stripPtr->scalelineColor);
     }
     if (stripPtr->scalelineGC != None) {
-	Tk_FreeGC(canvasPtr->display, stripPtr->scalelineGC);
+	Tk_FreeGC(display, stripPtr->scalelineGC);
     }
     if (stripPtr->textColor != NULL) {
         Tk_FreeColor(stripPtr->textColor);
     }
     if (stripPtr->textGC != None) {
-        Tk_FreeGC(canvasPtr->display, stripPtr->textGC);
+        Tk_FreeGC(display, stripPtr->textGC);
     }
 }
 
@@ -554,8 +572,8 @@ DeleteStripchart(canvasPtr, itemPtr)
  */
 
 static void
-ComputeStripchartBbox(canvasPtr, stripPtr)
-    register Tk_Canvas *canvasPtr;	/* Canvas that contains item. */
+ComputeStripchartBbox(canvas, stripPtr)
+    register Tk_Canvas canvas;		/* Canvas that contains item. */
     StripchartItem *stripPtr;		/* Item whose bbox is to be
 					 * recomputed. */
 {
@@ -602,8 +620,8 @@ ComputeStripchartBbox(canvasPtr, stripPtr)
  */
 
 static void
-FillStripchart(canvasPtr, itemPtr, coordPtr, drawable)
-    register Tk_Canvas *canvasPtr;	/* Canvas whose coordinate system
+FillStripchart(canvas, itemPtr, coordPtr, drawable)
+    register Tk_Canvas canvas;		/* Canvas whose coordinate system
 					 * is to be used for drawing. */
     Tk_Item *itemPtr;                   /* Item to fill. */
     double *coordPtr;			/* Array of coordinates for polygon:
@@ -633,8 +651,8 @@ FillStripchart(canvasPtr, itemPtr, coordPtr, drawable)
 
     for (i = 0, pPtr = pointPtr; i < stripPtr->numValues+2; i += 1, 
 	 coordPtr += 2, pPtr++) {
-	pPtr->x = SCREEN_X(canvasPtr, coordPtr[0]);
-	pPtr->y = SCREEN_Y(canvasPtr, coordPtr[1]);
+	Tk_CanvasDrawableCoords(canvas, coordPtr[0], coordPtr[1], 
+				&pPtr->x, &pPtr->y);
     }
 
     /*
@@ -643,12 +661,12 @@ FillStripchart(canvasPtr, itemPtr, coordPtr, drawable)
      */
 
     if (stripPtr->fillGC != NULL) {
-	XFillPolygon(Tk_Display(canvasPtr->tkwin), drawable, stripPtr->fillGC,
-		     pointPtr, stripPtr->numValues+2, Complex, 
-		     CoordModeOrigin);
+	XFillPolygon(Tk_Display(Tk_CanvasTkwin(canvas)), drawable, 
+		     stripPtr->fillGC, pointPtr, stripPtr->numValues+2, 
+		     Complex, CoordModeOrigin);
     }
     if (stripPtr->striplineGC != None) {
-	XDrawLines(Tk_Display(canvasPtr->tkwin), drawable, 
+	XDrawLines(Tk_Display(Tk_CanvasTkwin(canvas)), drawable, 
                    stripPtr->striplineGC, pointPtr, stripPtr->numValues+2,
 		   CoordModeOrigin);
     }
@@ -677,15 +695,18 @@ FillStripchart(canvasPtr, itemPtr, coordPtr, drawable)
  */
 
 static void
-DisplayStripchart(canvasPtr, itemPtr, drawable)
-    register Tk_Canvas *canvasPtr;	/* Canvas that contains item. */
+DisplayStripchart(canvas, itemPtr, display, drawable, x, y, width, height)
+    register Tk_Canvas canvas;		/* Canvas that contains item. */
     Tk_Item *itemPtr;			/* Item to be displayed. */
+    Display *display;                   /* Display on which to draw item. */  
     Drawable drawable;			/* Pixmap or window in which to draw
 					 * item. */
+    int x, y, width, height;            /* Describes region of canvas that
+                                         * must be redisplayed (not used). */
 {
     register StripchartItem *stripPtr = (StripchartItem *) itemPtr;
-    Display *display = Tk_Display(canvasPtr->tkwin);
-    int i, x1, y1, x2, y2;
+    int i;
+    short x1, y1, x2, y2;
 
     /*
      * Compute the screen coordinates of the bounding box for the item.
@@ -693,16 +714,18 @@ DisplayStripchart(canvasPtr, itemPtr, drawable)
      * X servers will die if it isn't.
      */
 
-    x1 = SCREEN_X(canvasPtr, stripPtr->bbox[0]);
-    y1 = SCREEN_Y(canvasPtr, stripPtr->bbox[1]);
-    x2 = SCREEN_X(canvasPtr, stripPtr->bbox[2]);
-    y2 = SCREEN_Y(canvasPtr, stripPtr->bbox[3]);
+    Tk_CanvasDrawableCoords(canvas, stripPtr->bbox[0], stripPtr->bbox[1],
+			    &x1, &y1);
+    Tk_CanvasDrawableCoords(canvas, stripPtr->bbox[2], stripPtr->bbox[3], 
+			    &x2, &y2);
+
     if (x2 <= x1) {
 	x2 = x1+1;
     }
     if (y2 <= y1) {
 	y2 = y1+1;
     }
+
     /* Display the background of the stripchart bounding box */
 
     if (stripPtr->rectGC != None) {
@@ -712,26 +735,41 @@ DisplayStripchart(canvasPtr, itemPtr, drawable)
 		       (unsigned int) (y2-y1-1));
     }
 		   
-    FillStripchart(canvasPtr, itemPtr, stripPtr->coordPtr, drawable);
+    FillStripchart(canvas, itemPtr, stripPtr->coordPtr, drawable);
     
-    /* Display the scaleline. */
+    /*
+     * Display the scalelines. Make sure that we only display lines
+     * that are actually displayable. Skip the first line to get 
+     * around rounding errors.
+     */
 
     if (stripPtr->scalelineGC != NULL){
 	if (stripPtr->scale > 1) {
-	    for (i = 1; i < stripPtr->scale; i++) {
+	    int lines = stripPtr->scale;
+	    if (lines > (y2-y1)) lines = y2-y1;
+	    for (i = 1; i < lines; i++) {
 		XDrawLine(display, drawable, stripPtr->scalelineGC,
-			  x1, y2-1-i*(y2-y1-2)/stripPtr->scale, 
-			  x2-1, y2-1-i*(y2-y1-2)/stripPtr->scale);
+			  x1,   y2 - i*(y2-y1)/lines, 
+			  x2-1, y2 - i*(y2-y1)/lines);
 	    }
 	}
     }
     
-    /* Display the border of the bounding box. */
+    /* 
+     * Display the border of the bounding box. 
+     */
     
     if (stripPtr->outlineGC != None) {
 	XDrawRectangle(display, drawable, stripPtr->outlineGC,
 		       x1, y1, (x2-x1), (y2-y1));
     }
+
+#if 0
+    if (stripPtr->textGC != None) {
+	XDrawString(display, drawable, stripPtr->textGC, x2 - x1, y2 + 10,
+		    "nase", 4);
+    }
+#endif
 }
 
 /*
@@ -756,8 +794,8 @@ DisplayStripchart(canvasPtr, itemPtr, drawable)
 
 	/* ARGSUSED */
 static double
-StripchartToPoint(canvasPtr, itemPtr, pointPtr)
-    Tk_Canvas *canvasPtr;	/* Canvas containing item. */
+StripchartToPoint(canvas, itemPtr, pointPtr)
+    Tk_Canvas canvas;		/* Canvas containing item. */
     Tk_Item *itemPtr;		/* Item to check against point. */
     double *pointPtr;		/* Pointer to x and y coordinates. */
 {
@@ -835,8 +873,8 @@ StripchartToPoint(canvasPtr, itemPtr, pointPtr)
 
 	/* ARGSUSED */
 static int
-StripchartToArea(canvasPtr, itemPtr, areaPtr)
-    Tk_Canvas *canvasPtr;	/* Canvas containing item. */
+StripchartToArea(canvas, itemPtr, areaPtr)
+    Tk_Canvas canvas;		/* Canvas containing item. */
     Tk_Item *itemPtr;		/* Item to check against stripchart. */
     double *areaPtr;		/* Pointer to array of four coordinates
 				 * (x1, y1, x2, y2) describing rectangular
@@ -886,8 +924,8 @@ StripchartToArea(canvasPtr, itemPtr, areaPtr)
  */
 
 static void
-ScaleStripchart(canvasPtr, itemPtr, originX, originY, scaleX, scaleY)
-    Tk_Canvas *canvasPtr;		/* Canvas containing stripchart. */
+ScaleStripchart(canvas, itemPtr, originX, originY, scaleX, scaleY)
+    Tk_Canvas canvas;			/* Canvas containing stripchart. */
     Tk_Item *itemPtr;			/* Stripchart to be scaled. */
     double originX, originY;		/* Origin about which to scale rect. */
     double scaleX;			/* Amount to scale in X direction. */
@@ -904,12 +942,10 @@ ScaleStripchart(canvasPtr, itemPtr, originX, originY, scaleX, scaleY)
 
     for (i = 0, coordPtr = stripPtr->coordPtr; i < stripPtr->numPoints;
 	    i++, coordPtr += 2) {
-	/*coordPtr[0] = originX + scaleX*(*coordPtr - originX);*/
 	coordPtr[1] = originY + scaleY*(coordPtr[1] - originY);
     }
-    ComputeStripchartBbox(canvasPtr, stripPtr);
-    StripchartValues(canvasPtr, itemPtr, 0, (char **) NULL);
-    
+    ComputeStripchartBbox(canvas, stripPtr);
+    StripchartValues(stripPtr->interp, canvas, itemPtr, 0, (char **) NULL);
 }
 
 /*
@@ -932,8 +968,8 @@ ScaleStripchart(canvasPtr, itemPtr, originX, originY, scaleX, scaleY)
  */
 
 static void
-TranslateStripchart(canvasPtr, itemPtr, deltaX, deltaY)
-    Tk_Canvas *canvasPtr;		/* Canvas containing item. */
+TranslateStripchart(canvas, itemPtr, deltaX, deltaY)
+    Tk_Canvas canvas;			/* Canvas containing item. */
     Tk_Item *itemPtr;			/* Item that is being moved. */
     double deltaX, deltaY;		/* Amount by which item is to be
 					 * moved. */
@@ -952,7 +988,7 @@ TranslateStripchart(canvasPtr, itemPtr, deltaX, deltaY)
 	*coordPtr += deltaX;
 	coordPtr[1] += deltaY;
     }
-    ComputeStripchartBbox(canvasPtr, stripPtr);
+    ComputeStripchartBbox(canvas, stripPtr);
 }
 
 /*
@@ -1002,7 +1038,7 @@ ParseStripchartValues(clientData, interp, tkwin, value, recordPtr, offset)
         return TCL_ERROR;
     }
 
-    if (StripchartValues(stripPtr->canvasPtr, (Tk_Item *) stripPtr, 
+    if (StripchartValues(interp, stripPtr->canvas, (Tk_Item *) stripPtr, 
 			 argc, argv) != TCL_OK) {
 	goto syntaxError;
     }
@@ -1028,8 +1064,9 @@ ParseStripchartValues(clientData, interp, tkwin, value, recordPtr, offset)
  */
 
 static int
-StripchartValues(canvasPtr, itemPtr, argc, argv) 
-    register Tk_Canvas *canvasPtr;      /* Canvas containing item. */
+StripchartValues(interp, canvas, itemPtr, argc, argv) 
+    Tcl_Interp *interp;			/* */
+    Tk_Canvas canvas;  			/* Canvas containing item. */
     Tk_Item *itemPtr;                   /* Item whose coordinates are to be
                                          * read or modified. */
     int argc;                           /* Number of coordinates supplied in
@@ -1038,15 +1075,16 @@ StripchartValues(canvasPtr, itemPtr, argc, argv)
                                          * x2, y2, ... */
 {    
     register StripchartItem *stripPtr = (StripchartItem *) itemPtr;
-    int i, j, x1, x2, y1, y2;
+    int i, j;
+    short x1, x2, y1, y2;
     int numPoints, numCoords, width, height, jump;
     double max, scaleValue;
     double *savePtr = NULL;
 
-    x1 = SCREEN_X(canvasPtr, stripPtr->bbox[0]);
-    x2 = SCREEN_X(canvasPtr, stripPtr->bbox[2]);    
-    y1 = SCREEN_Y(canvasPtr, stripPtr->bbox[1]);
-    y2 = SCREEN_Y(canvasPtr, stripPtr->bbox[3]);
+    Tk_CanvasDrawableCoords(canvas, stripPtr->bbox[0], stripPtr->bbox[1],
+			    &x1, &y1);
+    Tk_CanvasDrawableCoords(canvas, stripPtr->bbox[2], stripPtr->bbox[3], 
+			    &x2, &y2);
     height = (y2 == y1) ? 0 : y2-y1-1;
     width = (x2 == x1) ? 0 : x2-x1-1;
     numPoints = width+2;
@@ -1117,14 +1155,14 @@ StripchartValues(canvasPtr, itemPtr, argc, argv)
 	    }
 	}
 
-	if (TkGetCanvasCoord(canvasPtr, argv[i],
+	if (Tk_CanvasGetCoord(interp, canvas, argv[i],
 		&stripPtr->shiftPtr[stripPtr->numValues]) != TCL_OK) {
 	    return TCL_ERROR;
 	}
 	stripPtr->numValues++;
     }
     
-    ckfree((char *) argv);
+    if (argv) ckfree((char *) argv);
 
     /* Clip negative values. */
 
@@ -1166,24 +1204,16 @@ StripchartValues(canvasPtr, itemPtr, argc, argv)
     /* Find the right scale. */
     
     scaleValue = stripPtr->scaleValue;
-    stripPtr->scaledUp = 0;
 
     stripPtr->scale = 1;
     if (max/stripPtr->scale > scaleValue) {
 	stripPtr->scale = (int) (max / scaleValue) + 1;
     }
     
-    /* If we would have to draw to many scalelines, we change the 
-     * scalevalue. */
-
-   /* while(stripPtr->scale > 1 && stripPtr->scale > height/2) {
-	scaleValue *= 10;
-	stripPtr->scale = (int) (max / scaleValue) + 1;
-	stripPtr->scaledUp = 1;
-    }*/
-	
-    /* Adapt the y-coordinates to the actual scale and to the actual
-     * bounding box. */
+    /* 
+     * Adapt the y-coordinates to the actual scale and to the actual
+     * bounding box. 
+     */
 
     for (i = 0; i < numCoords; i+=2) {
 	stripPtr->coordPtr[i+1] = 
@@ -1235,8 +1265,7 @@ PrintStripchartValues(clientData, tkwin, recordPtr, offset, freeProcPtr)
     Tcl_DStringInit(&buffer);
 
     for (i = 0; i < stripPtr->numValues; i++) {
-	Tcl_PrintDouble(stripPtr->canvasPtr->interp, 
-			stripPtr->shiftPtr[i], tmp);
+	Tcl_PrintDouble(stripPtr->interp, stripPtr->shiftPtr[i], tmp);
 	Tcl_DStringAppendElement(&buffer, tmp);
     }
 
@@ -1269,21 +1298,23 @@ PrintStripchartValues(clientData, tkwin, recordPtr, offset, freeProcPtr)
  */
 
 static int
-StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
-    Tk_Canvas *canvasPtr;		/* Information about overall canvas. */
+StripchartToPostscript(interp, canvas, itemPtr, prepass)
+    Tcl_Interp *interp;                 /* Leave Postscript or error message
+                                         * here. */
+    Tk_Canvas canvas;			/* Information about overall canvas. */
     Tk_Item *itemPtr;			/* Item for which Postscript is
 					 * wanted. */
-    Tk_PostscriptInfo *psInfoPtr;	/* Information about the Postscript;
-					 * must be passed back to Postscript
-					 * utility procedures. */
+    int prepass;                        /* 1 means this is a prepass to
+                                         * collect font information;  0 means
+                                         * final Postscript is being created. */
 {
     register StripchartItem *stripPtr = (StripchartItem *) itemPtr;
     char pathCmd1[500], pathCmd2[500], string[100];
     double y1, y2;
     int dash, i;
 
-    y1 = TkCanvPsY(psInfoPtr, stripPtr->bbox[1]);
-    y2 = TkCanvPsY(psInfoPtr, stripPtr->bbox[3]);
+    y1 = Tk_CanvasPsY(canvas, stripPtr->bbox[1]);
+    y2 = Tk_CanvasPsY(canvas, stripPtr->bbox[3]);
 
     /*
      * Generate a string that creates a path for the stripchart.
@@ -1301,12 +1332,12 @@ StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
      */
 
     if (stripPtr->rectColor != NULL) {
-        Tcl_AppendResult(canvasPtr->interp, pathCmd1, (char *) NULL);
-        if (TkCanvPsColor(canvasPtr, psInfoPtr, stripPtr->rectColor)
+        Tcl_AppendResult(interp, pathCmd1, (char *) NULL);
+        if (Tk_CanvasPsColor(interp, canvas, stripPtr->rectColor)
                 != TCL_OK) {
             return TCL_ERROR;
         }
-      	Tcl_AppendResult(canvasPtr->interp, "fill\n", (char *) NULL);
+      	Tcl_AppendResult(interp, "fill\n", (char *) NULL);
     }
 
 
@@ -1318,19 +1349,18 @@ StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
      * Generate a path for the polygon's outline.
      */
 
-    TkCanvPsPath(canvasPtr->interp, stripPtr->coordPtr,
-		 stripPtr->numPoints, psInfoPtr);
+    Tk_CanvasPsPath(interp, canvas, stripPtr->coordPtr, stripPtr->numPoints);
 
     /*
      * Fill the area of the polygon.
      */
 
     if (stripPtr->stripColor != NULL) {
-	if (TkCanvPsColor(canvasPtr, psInfoPtr, stripPtr->stripColor) 
+	if (Tk_CanvasPsColor(interp, canvas, stripPtr->stripColor) 
 	    != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	Tcl_AppendResult(canvasPtr->interp, "eofill\n", (char *) NULL);
+	Tcl_AppendResult(interp, "eofill\n", (char *) NULL);
     }
 
     /*
@@ -1338,14 +1368,14 @@ StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
      */
 
     if (stripPtr->striplineColor != NULL) {
-	TkCanvPsPath(canvasPtr->interp, stripPtr->coordPtr,
-		     stripPtr->numPoints, psInfoPtr);
+	Tk_CanvasPsPath(interp, canvas, stripPtr->coordPtr, 
+			stripPtr->numPoints);
 	
-	if (TkCanvPsColor(canvasPtr, psInfoPtr, stripPtr->striplineColor)
+	if (Tk_CanvasPsColor(interp, canvas, stripPtr->striplineColor)
 	    != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	Tcl_AppendResult(canvasPtr->interp, "stroke\n", (char *) NULL);
+	Tcl_AppendResult(interp, "stroke\n", (char *) NULL);
     }
 
     /*
@@ -1355,7 +1385,7 @@ StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
     /* Generate a path for the scaleline. */
 
     for (i = 1; i < stripPtr->scale; i++) {
-	y1 = TkCanvPsY(psInfoPtr, (stripPtr->bbox[3]-1-i*
+	y1 = Tk_CanvasPsY(canvas, (stripPtr->bbox[3]-1-i*
 	         (stripPtr->bbox[3]-stripPtr->bbox[1]-2)/stripPtr->scale));
 
 	sprintf(pathCmd2, "%.15g %.15g moveto %.15g 0 rlineto\n",
@@ -1363,8 +1393,7 @@ StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
 		stripPtr->bbox[2]-stripPtr->bbox[0]);
 
 	if (stripPtr->scalelineColor != NULL) {
-	    Tcl_AppendResult(canvasPtr->interp, pathCmd2, 
-			     (char *) NULL);
+	    Tcl_AppendResult(interp, pathCmd2, (char *) NULL);
 	    if (stripPtr->scalelineStyle > 0) {
 		dash = stripPtr->scalelineStyle;
 		sprintf(string,
@@ -1372,16 +1401,14 @@ StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
 	    } else {
 		sprintf(string, " 0 setlinejoin 2 setlinecap [] 0 setdash\n");
 	    }
-	    Tcl_AppendResult(canvasPtr->interp, string, 
-			     (char *) NULL);
+	    Tcl_AppendResult(interp, string, (char *) NULL);
 	    
-	    if (TkCanvPsColor(canvasPtr, psInfoPtr, stripPtr->scalelineColor)
+	    if (Tk_CanvasPsColor(interp, canvas, stripPtr->scalelineColor)
                 != TCL_OK) {
 		return TCL_ERROR;
 	    }
-	    Tcl_AppendResult(canvasPtr->interp, "stroke\n", (char *) NULL);
-	    Tcl_AppendResult(canvasPtr->interp, 
-			     "[] 0 setdash\n", (char *) NULL);
+	    Tcl_AppendResult(interp, "stroke\n", (char *) NULL);
+	    Tcl_AppendResult(interp, "[] 0 setdash\n", (char *) NULL);
 	}
     }
 
@@ -1390,14 +1417,14 @@ StripchartToPostscript(canvasPtr, itemPtr, psInfoPtr)
      */
 
     if (stripPtr->outlineColor != NULL) {
-        Tcl_AppendResult(canvasPtr->interp, pathCmd1, (char *) NULL);
-        Tcl_AppendResult(canvasPtr->interp,
+        Tcl_AppendResult(interp, pathCmd1, (char *) NULL);
+        Tcl_AppendResult(interp,
                 " 0 setlinejoin 2 setlinecap [] 0 setdash\n", (char *) NULL);
-	if (TkCanvPsColor(canvasPtr, psInfoPtr, stripPtr->outlineColor)
+	if (Tk_CanvasPsColor(interp, canvas, stripPtr->outlineColor)
                 != TCL_OK) {
             return TCL_ERROR;
         }
-        Tcl_AppendResult(canvasPtr->interp, "stroke\n", (char *) NULL);
+        Tcl_AppendResult(interp, "stroke\n", (char *) NULL);
     }
 
     return TCL_OK;
